# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2015, Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from ..geometry import create_cube
from ..gloo import set_state
from .mesh import MeshVisual


class CubeVisual(MeshVisual):
    """Visual that displays a cube or cuboid

    Parameters
    ----------
    size : float or tuple
        The size of the cuboid. A float gives a cube, whereas tuples may
        specify the size of each axis (x, y, z) independently.
    vertex_colors : ndarray
        Same as for `MeshVisual` class. See `create_cube` for vertex ordering.
    face_colors : ndarray
        Same as for `MeshVisual` class. See `create_cube` for vertex ordering.
    color : Color
        The `Color` to use when drawing the cube faces.
    edge_color : tuple or Color
        The `Color` to use when drawing the cube edges. If `None`, then no
        cube edges are drawn.
    """
    def __init__(self, size=1.0, vertex_colors=None, face_colors=None,
                 color=(0.5, 0.5, 1, 1), edge_color=None):
        vertices, filled_indices, outline_indices = create_cube()
        vertices['position'] *= size

        MeshVisual.__init__(self, vertices['position'], filled_indices,
                            vertex_colors, face_colors, color)
        if edge_color:
            self._outline = MeshVisual(vertices['position'], outline_indices,
                                       color=edge_color, mode='lines')
        else:
            self._outline = None

    def draw(self, transforms):
        """Draw the visual

        Parameters
        ----------
        transforms : instance of TransformSystem
            The transforms to use.
        """
        MeshVisual.draw(self, transforms)
        if self._outline:
            set_state(polygon_offset=(1, 1), polygon_offset_fill=True)
            self._outline.draw(transforms)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from .node import Node
from .systems import DrawingSystem, MouseInputSystem


class SubScene(Node):
    """ A subscene with entities.

    A subscene can be a child of a Canvas or a ViewBox. It is a
    placeholder for the transform to make the sub scene appear as
    intended. It's transform cannot be mannually set, but its set
    automatically and is based on three components:

      * Viewbox transform: a scale and translation to make the subscene
        be displayed in the boundaries of the viewbox.
      * Projection transform: the camera projection, e.g. perspective
      * position transform: the inverse of the transform from this
        subscene to the camera. This takes care of position and
        orientation of the view.

    TODO: should camera, lights, etc. be properties of the subscene or the
    viewbox? I think of the scene. In that way canvas.scene can simply
    be a subscene.
    """

    def __init__(self, **kwargs):
        Node.__init__(self, **kwargs)

        # Initialize systems
        self._systems = {}
        self._systems['draw'] = DrawingSystem()
        self._systems['mouse'] = MouseInputSystem()
        self._drawing = False
    
    def draw(self, event):
        # Temporary workaround to avoid infinite recursion. A better solution
        # would be for ViewBox and Canvas to handle the systems, rather than
        # subscene.
        if self._drawing:
            return
        
        # Invoke our drawing system
        try:
            self._drawing = True
            self.process_system(event, 'draw')
        finally:
            self._drawing = False
    
    def _process_mouse_event(self, event):
        self.process_system(event, 'mouse') 

    def process_system(self, event, system_name):
        """ Process a system.
        """
        self._systems[system_name].process(event, self)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from vispy.io import read_png, load_data_file
from vispy.testing import (has_matplotlib, requires_application,
                           run_tests_if_main, assert_raises)
import vispy.mpl_plot as plt


@requires_application()
def test_show_vispy():
    """Some basic tests of show_vispy"""
    if has_matplotlib():
        n = 200
        t = np.arange(n)
        noise = np.random.RandomState(0).randn(n)
        # Need, image, markers, line, axes, figure
        plt.figure()
        ax = plt.subplot(211)
        ax.imshow(read_png(load_data_file('pyplot/logo.png')))
        ax = plt.subplot(212)
        ax.plot(t, noise, 'ko-')
        plt.draw()
        canvases = plt.show()
        canvases[0].close()
    else:
        assert_raises(ImportError, plt.show)


run_tests_if_main()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ..scene import (Image, LinePlot, Volume, Mesh, Histogram,
                     Spectrogram, ViewBox, PanZoomCamera, TurntableCamera)
from ..io import read_mesh
from ..geometry import MeshData

__all__ = ['PlotWidget']


class PlotWidget(ViewBox):
    """Widget to facilitate plotting

    Parameters
    ----------
    *args : arguments
        Arguments passed to the `ViewBox` super class.
    **kwargs : keywoard arguments
        Keyword arguments passed to the `ViewBox` super class.

    Notes
    -----
    This class is typically instantiated implicitly by a `Figure`
    instance, e.g., by doing ``fig[0, 0]``.

    See Also
    --------
    """
    def __init__(self, *args, **kwargs):
        super(PlotWidget, self).__init__(*args, **kwargs)
        self._camera_set = False

    def _set_camera(self, cls, *args, **kwargs):
        if not self._camera_set:
            self._camera_set = True
            self.camera = cls(*args, **kwargs)
            self.camera.set_range(margin=0)

    def histogram(self, data, bins=10, color='w', orientation='h'):
        """Calculate and show a histogram of data

        Parameters
        ----------
        data : array-like
            Data to histogram. Currently only 1D data is supported.
        bins : int | array-like
            Number of bins, or bin edges.
        color : instance of Color
            Color of the histogram.
        orientation : {'h', 'v'}
            Orientation of the histogram.

        Returns
        -------
        hist : instance of Polygon
            The histogram polygon.
        """
        hist = Histogram(data, bins, color, orientation)
        self.add(hist)
        self._set_camera(PanZoomCamera)
        return hist

    def image(self, data, cmap='cubehelix', clim='auto'):
        """Show an image

        Parameters
        ----------
        data : ndarray
            Should have shape (N, M), (N, M, 3) or (N, M, 4).
        cmap : str
            Colormap name.
        clim : str | tuple
            Colormap limits. Should be ``'auto'`` or a two-element tuple of
            min and max values.

        Returns
        -------
        image : instance of Image
            The image.

        Notes
        -----
        The colormap is only used if the image pixels are scalars.
        """
        image = Image(data, cmap=cmap, clim=clim)
        self.add(image)
        self._set_camera(PanZoomCamera, aspect=1)
        return image

    def mesh(self, vertices=None, faces=None, vertex_colors=None,
             face_colors=None, color=(0.5, 0.5, 1.), fname=None,
             meshdata=None):
        """Show a 3D mesh

        Parameters
        ----------
        vertices : array
            Vertices.
        faces : array | None
            Face definitions.
        vertex_colors : array | None
            Vertex colors.
        face_colors : array | None
            Face colors.
        color : instance of Color
            Color to use.
        fname : str | None
            Filename to load. If not None, then vertices, faces, and meshdata
            must be None.
        meshdata : MeshData | None
            Meshdata to use. If not None, then vertices, faces, and fname
            must be None.

        Returns
        -------
        mesh : instance of Mesh
            The mesh.
        """
        if fname is not None:
            if not all(x is None for x in (vertices, faces, meshdata)):
                raise ValueError('vertices, faces, and meshdata must be None '
                                 'if fname is not None')
            vertices, faces = read_mesh(fname)[:2]
        if meshdata is not None:
            if not all(x is None for x in (vertices, faces, fname)):
                raise ValueError('vertices, faces, and fname must be None if '
                                 'fname is not None')
        else:
            meshdata = MeshData(vertices, faces)
        mesh = Mesh(meshdata=meshdata, vertex_colors=vertex_colors,
                    face_colors=face_colors, color=color, shading='smooth')
        self.add(mesh)
        self._set_camera(TurntableCamera, azimuth=0, elevation=0)
        return mesh

    def plot(self, data, color='k', symbol='o', line_kind='-', width=1.,
             marker_size=0., edge_color='k', face_color='k', edge_width=1.):
        """Plot a series of data using lines and markers

        Parameters
        ----------
        data : array | two arrays
            Arguments can be passed as ``(Y,)``, ``(X, Y)`` or
            ``np.array((X, Y))``.
        color : instance of Color
            Color of the line.
        symbol : str
            Marker symbol to use.
        line_kind : str
            Kind of line to draw. For now, only solid lines (``'-'``)
            are supported.
        width : float
            Line width.
        marker_size : float
            Marker size. If `size == 0` markers will not be shown.
        edge_color : instance of Color
            Color of the marker edge.
        face_color : instance of Color
            Color of the marker face.
        edge_width : float
            Edge width of the marker.

        Returns
        -------
        line : instance of LinePlot
            The line plot.

        See also
        --------
        marker_types, LinePlot
        """
        line = LinePlot(data, connect='strip', color=color, symbol=symbol,
                        line_kind=line_kind, width=width,
                        marker_size=marker_size, edge_color=edge_color,
                        face_color=face_color, edge_width=edge_width)
        self.add(line)
        self._set_camera(PanZoomCamera)
        return line

    def spectrogram(self, x, n_fft=256, step=None, fs=1., window='hann',
                    color_scale='log', cmap='cubehelix', clim='auto'):
        """Calculate and show a spectrogram

        Parameters
        ----------
        x : array-like
            1D signal to operate on. ``If len(x) < n_fft``, x will be
            zero-padded to length ``n_fft``.
        n_fft : int
            Number of FFT points. Much faster for powers of two.
        step : int | None
            Step size between calculations. If None, ``n_fft // 2``
            will be used.
        fs : float
            The sample rate of the data.
        window : str | None
            Window function to use. Can be ``'hann'`` for Hann window, or None
            for no windowing.
        color_scale : {'linear', 'log'}
            Scale to apply to the result of the STFT.
            ``'log'`` will use ``10 * log10(power)``.
        cmap : str
            Colormap name.
        clim : str | tuple
            Colormap limits. Should be ``'auto'`` or a two-element tuple of
            min and max values.

        Returns
        -------
        spec : instance of Spectrogram
            The spectrogram.

        See also
        --------
        Image
        """
        # XXX once we have axes, we should use "fft_freqs", too
        spec = Spectrogram(x, n_fft, step, fs, window,
                           color_scale, cmap, clim)
        self.add(spec)
        self._set_camera(PanZoomCamera)
        return spec

    def volume(self, vol, clim=None, method='mip', threshold=None,
               cmap='grays'):
        """Show a 3D volume

        Parameters
        ----------
        vol : ndarray
            Volume to render.
        clim : tuple of two floats | None
            The contrast limits. The values in the volume are mapped to
            black and white corresponding to these values. Default maps
            between min and max.
        method : {'mip', 'iso', 'translucent', 'additive'}
            The render style to use. See corresponding docs for details.
            Default 'mip'.
        threshold : float
            The threshold to use for the isosurafce render style. By default
            the mean of the given volume is used.
        cmap : str
            The colormap to use.

        Returns
        -------
        volume : instance of Volume
            The volume visualization.

        See also
        --------
        Volume
        """
        volume = Volume(vol, clim, method, threshold, cmap=cmap)
        self.add(volume)
        self._set_camera(TurntableCamera, fov=30.)
        return volume



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2015, Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
# Author: Nicolas P .Rougier
# Date:   04/03/2014
# -----------------------------------------------------------------------------
from __future__ import division

import numpy as np

from .meshdata import MeshData


def create_cube():
    """ Generate vertices & indices for a filled and outlined cube

    Returns
    -------
    vertices : array
        Array of vertices suitable for use as a VertexBuffer.
    filled : array
        Indices to use to produce a filled cube.
    outline : array
        Indices to use to produce an outline of the cube.
    """
    vtype = [('position', np.float32, 3),
             ('texcoord', np.float32, 2),
             ('normal', np.float32, 3),
             ('color',    np.float32, 4)]
    itype = np.uint32

    # Vertices positions
    p = np.array([[1, 1, 1], [-1, 1, 1], [-1, -1, 1], [1, -1, 1],
                  [1, -1, -1], [1, 1, -1], [-1, 1, -1], [-1, -1, -1]])

    # Face Normals
    n = np.array([[0, 0, 1], [1, 0, 0], [0, 1, 0],
                  [-1, 0, 1], [0, -1, 0], [0, 0, -1]])

    # Vertice colors
    c = np.array([[1, 1, 1, 1], [0, 1, 1, 1], [0, 0, 1, 1], [1, 0, 1, 1],
                  [1, 0, 0, 1], [1, 1, 0, 1], [0, 1, 0, 1], [0, 0, 0, 1]])

    # Texture coords
    t = np.array([[0, 0], [0, 1], [1, 1], [1, 0]])

    faces_p = [0, 1, 2, 3,
               0, 3, 4, 5,
               0, 5, 6, 1,
               1, 6, 7, 2,
               7, 4, 3, 2,
               4, 7, 6, 5]
    faces_c = [0, 1, 2, 3,
               0, 3, 4, 5,
               0, 5, 6, 1,
               1, 6, 7, 2,
               7, 4, 3, 2,
               4, 7, 6, 5]
    faces_n = [0, 0, 0, 0,
               1, 1, 1, 1,
               2, 2, 2, 2,
               3, 3, 3, 3,
               4, 4, 4, 4,
               5, 5, 5, 5]
    faces_t = [0, 1, 2, 3,
               0, 1, 2, 3,
               0, 1, 2, 3,
               3, 2, 1, 0,
               0, 1, 2, 3,
               0, 1, 2, 3]

    vertices = np.zeros(24, vtype)
    vertices['position'] = p[faces_p]
    vertices['normal'] = n[faces_n]
    vertices['color'] = c[faces_c]
    vertices['texcoord'] = t[faces_t]

    filled = np.resize(
        np.array([0, 1, 2, 0, 2, 3], dtype=itype), 6 * (2 * 3))
    filled += np.repeat(4 * np.arange(6, dtype=itype), 6)
    filled = filled.reshape((len(filled) // 3, 3))

    outline = np.resize(
        np.array([0, 1, 1, 2, 2, 3, 3, 0], dtype=itype), 6 * (2 * 4))
    outline += np.repeat(4 * np.arange(6, dtype=itype), 8)

    return vertices, filled, outline


def create_sphere(rows, cols, radius=1.0, offset=True):
    """Create a sphere

    Parameters
    ----------
    rows : int
        Number of rows.
    cols : int
        Number of columns.
    radius : float
        Sphere radius.
    offset : bool
        Rotate each row by half a column.

    Returns
    -------
    sphere : MeshData
        Vertices and faces computed for a spherical surface.
    """
    verts = np.empty((rows+1, cols, 3), dtype=np.float32)

    # compute vertices
    phi = (np.arange(rows+1) * np.pi / rows).reshape(rows+1, 1)
    s = radius * np.sin(phi)
    verts[..., 2] = radius * np.cos(phi)
    th = ((np.arange(cols) * 2 * np.pi / cols).reshape(1, cols))
    if offset:
        # rotate each row by 1/2 column
        th = th + ((np.pi / cols) * np.arange(rows+1).reshape(rows+1, 1))
    verts[..., 0] = s * np.cos(th)
    verts[..., 1] = s * np.sin(th)
    # remove redundant vertices from top and bottom
    verts = verts.reshape((rows+1)*cols, 3)[cols-1:-(cols-1)]

    # compute faces
    faces = np.empty((rows*cols*2, 3), dtype=np.uint32)
    rowtemplate1 = (((np.arange(cols).reshape(cols, 1) +
                      np.array([[1, 0, 0]])) % cols) +
                    np.array([[0, 0, cols]]))
    rowtemplate2 = (((np.arange(cols).reshape(cols, 1) +
                      np.array([[1, 0, 1]])) % cols) +
                    np.array([[0, cols, cols]]))
    for row in range(rows):
        start = row * cols * 2
        faces[start:start+cols] = rowtemplate1 + row * cols
        faces[start+cols:start+(cols*2)] = rowtemplate2 + row * cols
    # cut off zero-area triangles at top and bottom
    faces = faces[cols:-cols]

    # adjust for redundant vertices that were removed from top and bottom
    vmin = cols-1
    faces[faces < vmin] = vmin
    faces -= vmin
    vmax = verts.shape[0]-1
    faces[faces > vmax] = vmax
    return MeshData(vertices=verts, faces=faces)


def create_cylinder(rows, cols, radius=[1.0, 1.0], length=1.0, offset=False):
    """Create a cylinder

    Parameters
    ----------
    rows : int
        Number of rows.
    cols : int
        Number of columns.
    radius : tuple of float
        Cylinder radii.
    length : float
        Length of the cylinder.
    offset : bool
        Rotate each row by half a column.

    Returns
    -------
    cylinder : MeshData
        Vertices and faces computed for a cylindrical surface.
    """
    verts = np.empty((rows+1, cols, 3), dtype=np.float32)
    if isinstance(radius, int):
        radius = [radius, radius]  # convert to list
    # compute vertices
    th = np.linspace(2 * np.pi, 0, cols).reshape(1, cols)
    # radius as a function of z
    r = np.linspace(radius[0], radius[1], num=rows+1,
                    endpoint=True).reshape(rows+1, 1)
    verts[..., 2] = np.linspace(0, length, num=rows+1,
                                endpoint=True).reshape(rows+1, 1)  # z
    if offset:
        # rotate each row by 1/2 column
        th = th + ((np.pi / cols) * np.arange(rows+1).reshape(rows+1, 1))
    verts[..., 0] = r * np.cos(th)  # x = r cos(th)
    verts[..., 1] = r * np.sin(th)  # y = r sin(th)
    # just reshape: no redundant vertices...
    verts = verts.reshape((rows+1)*cols, 3)
    # compute faces
    faces = np.empty((rows*cols*2, 3), dtype=np.uint32)
    rowtemplate1 = (((np.arange(cols).reshape(cols, 1) +
                      np.array([[0, 1, 0]])) % cols) +
                    np.array([[0, 0, cols]]))
    rowtemplate2 = (((np.arange(cols).reshape(cols, 1) +
                      np.array([[0, 1, 1]])) % cols) +
                    np.array([[cols, 0, cols]]))
    for row in range(rows):
        start = row * cols * 2
        faces[start:start+cols] = rowtemplate1 + row * cols
        faces[start+cols:start+(cols*2)] = rowtemplate2 + row * cols

    return MeshData(vertices=verts, faces=faces)


def create_cone(cols, radius=1.0, length=1.0):
    """Create a cone

    Parameters
    ----------
    cols : int
        Number of faces.
    radius : float
        Base cone radius.
    length : float
        Length of the cone.

    Returns
    -------
    cone : MeshData
        Vertices and faces computed for a cone surface.
    """
    verts = np.empty((cols+1, 3), dtype=np.float32)
    # compute vertexes
    th = np.linspace(2 * np.pi, 0, cols+1).reshape(1, cols+1)
    verts[:-1, 2] = 0.0
    verts[:-1, 0] = radius * np.cos(th[0, :-1])  # x = r cos(th)
    verts[:-1, 1] = radius * np.sin(th[0, :-1])  # y = r sin(th)
    # Add the extremity
    verts[-1, 0] = 0.0
    verts[-1, 1] = 0.0
    verts[-1, 2] = length
    verts = verts.reshape((cols+1), 3)  # just reshape: no redundant vertices
    # compute faces
    faces = np.empty((cols, 3), dtype=np.uint32)
    template = np.array([[0, 1]])
    for pos in range(cols):
        faces[pos, :-1] = template + pos
    faces[:, 2] = cols
    faces[-1, 1] = 0

    return MeshData(vertices=verts, faces=faces)


def create_arrow(rows, cols, radius=0.1, length=1.0,
                 cone_radius=None, cone_length=None):
    """Create a 3D arrow using a cylinder plus cone

    Parameters
    ----------
    rows : int
        Number of rows.
    cols : int
        Number of columns.
    radius : float
        Base cylinder radius.
    length : float
        Length of the arrow.
    cone_radius : float
        Radius of the cone base.
           If None, then this defaults to 2x the cylinder radius.
    cone_length : float
        Length of the cone.
           If None, then this defaults to 1/3 of the arrow length.

    Returns
    -------
    arrow : MeshData
        Vertices and faces computed for a cone surface.
    """
    # create the cylinder
    md_cyl = None
    if cone_radius is None:
        cone_radius = radius*2.0
    if cone_length is None:
        con_L = length/3.0
        cyl_L = length*2.0/3.0
    else:
        cyl_L = max(0, length - cone_length)
        con_L = min(cone_length, length)
    if cyl_L != 0:
        md_cyl = create_cylinder(rows, cols, radius=[radius, radius],
                                 length=cyl_L)
    # create the cone
    md_con = create_cone(cols, radius=cone_radius, length=con_L)
    verts = md_con.get_vertices()
    nbr_verts_con = verts.size//3
    faces = md_con.get_faces()
    if md_cyl is not None:
        trans = np.array([[0.0, 0.0, cyl_L]])
        verts = np.vstack((verts+trans, md_cyl.get_vertices()))
        faces = np.vstack((faces, md_cyl.get_faces()+nbr_verts_con))

    return MeshData(vertices=verts, faces=faces)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np
from os import path as op

from ..util import load_data_file

# This is the package data dir, not the dir for config, etc.
DATA_DIR = op.join(op.dirname(__file__), '_data')


def load_iris():
    """Load the iris dataset

    Returns
    -------
    iris : NpzFile
        data['data'] : a (150, 4) NumPy array with the iris' features
        data['group'] : a (150,) NumPy array with the iris' group
    """
    return np.load(load_data_file('iris/iris.npz',
                                  force_download='2014-09-04'))


def load_crate():
    """Load an image of a crate

    Returns
    -------
    crate : array
        256x256x3 crate image.
    """
    return np.load(load_data_file('orig/crate.npz'))['crate']



from ..shaders import Function


class VisualComponent(object):
    """
    Base for classes that encapsulate some modular component of a Visual.

    These define Functions for extending the shader code as well as an
    activate() method that inserts these Functions into a program.

    VisualComponents may be considered friends of the Visual they are attached
    to; often they will need to access internal data structures of the Visual
    to make decisions about constructing shader components.
    """

    DRAW_PRE_INDEXED = 1
    DRAW_UNINDEXED = 2

    # Maps {'program_hook': 'GLSL code'}
    SHADERS = {}

    # List of shaders to automatically attach to the visual's program.
    # If None, then all shaders are attached.
    AUTO_ATTACH = None

    def __init__(self, visual=None):
        self._visual = None
        if visual is not None:
            self._attach(visual)

        self._funcs = dict([(name, Function(code))
                            for name, code in self.SHADERS.items()])

        # components that are required by this component
        self._deps = []

        # only detach when count drops to 0;
        # this is like a reference count for component dependencies.
        self._attach_count = 0

    @property
    def visual(self):
        """The Visual that this component is attached to."""
        return self._visual

    def _attach(self, visual):
        """Attach this component to a Visual. This should be called by the
        Visual itself.

        A component may only be attached to a single Visual. However, it may
        be attached _multiple times_ to the same visual.

        The default implementation of this method calls
        self._auto_attach_shaders() to generate the list of shader callbacks
        that should be added to the Visual's program.
        """
        if visual is not self._visual and self._visual is not None:
            raise Exception("Cannot attach component %s to %s; already "
                            "attached to %s" % (self, visual, self._visual))
        self._visual = visual
        self._attach_count += 1
        for hook in self._auto_attach_shaders():
            func = self._funcs[hook]
            try:
                visual._program.vert.add_callback(hook, func)
            except KeyError:
                visual._program.frag.add_callback(hook, func)
                
        for comp in self._deps:
            comp._attach(visual)

    def _detach(self):
        """Detach this component from its Visual. This should be called by the
        visual itself.

        If the component was attached
        multiple times, it must be detached the same number of times.
        """
        if self._attach_count == 0:
            raise Exception("Cannot detach component %s; not attached." % self)

        self._attach_count -= 1
        if self._attach_count == 0:
            for hook in self._auto_attach_shaders():
                func = self._funcs[hook]
                try:
                    self._visual._program.vert.remove_callback(hook, func)
                except KeyError:
                    self._visual._program.frag.remove_callback(hook, func)
            self._visual = None
            for comp in self._deps:
                comp._detach()

    def _auto_attach_shaders(self):
        """
        Return a list of shaders to automatically attach/detach
        """
        if self.AUTO_ATTACH is None:
            return self._funcs.keys()
        else:
            return self.AUTO_ATTACH

    @property
    def supported_draw_modes(self):
        """
        A set of the draw modes (either DRAW_PRE_INDEXED, DRAW_UNINDEXED, or
        both) currently supported by this component.

        DRAW_PRE_INDEXED indicates that the component may be used when the
        program uses an array of indices to determine the order of elements to
        draw from its vertex buffers (using glDrawElements).

        DRAW_UNINDEXED indicates that the component may be used when the
        program will not use an array of indices; rather, vertex buffers are
        processed in the order they appear in the buffer (using glDrawArrays).

        By default, this method returns a tuple with both values. Components
        that only support one mode must override this method.
        """
        # TODO: This should be expanded to include other questions, such as
        # whether the component supports geometry shaders.
        return set([self.DRAW_PRE_INDEXED, self.DRAW_UNINDEXED])

    def update(self):
        """
        Inform the attached visual that this component has changed.
        """
        if self.visual is not None:
            self.visual.update()

    def activate(self, program):
        """
        *program* is about to draw; attach to *program* all functions and
        data required by this component.
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from .. import gloo
from .visual import Visual
from .shaders import ModularProgram
from .transforms import TransformCache

VERT = """
attribute vec2 pos;
varying vec4 v_pos;
void main() {
    v_pos = vec4(pos, 0, 1);
    gl_Position = v_pos;
}
"""

FRAG = """
varying vec4 v_pos;
uniform vec2 scale;

void main() {
    vec4 px_pos = $map_nd_to_doc(v_pos);

    // Compute vectors representing width, height of pixel in local coords
    float s = 1.;
    vec4 local_pos = $map_doc_to_local(px_pos);
    vec4 dx = $map_doc_to_local(px_pos + vec4(1.0 / s, 0, 0, 0)) - local_pos;
    vec4 dy = $map_doc_to_local(px_pos + vec4(0, 1.0 / s, 0, 0)) - local_pos;

    // Pixel length along each axis, rounded to the nearest power of 10
    vec2 px = s * vec2(abs(dx.x) + abs(dy.x), abs(dx.y) + abs(dy.y));
    float log10 = log(10.0);
    float sx = pow(10.0, floor(log(px.x) / log10)+1) * scale.x;
    float sy = pow(10.0, floor(log(px.y) / log10)+1) * scale.y;

    float max_alpha = 0.6;
    float x_alpha = 0.0;

    if (mod(local_pos.x, 1000 * sx) < px.x) {
        x_alpha = clamp(1 * sx/px.x, 0, max_alpha);
    }
    else if (mod(local_pos.x, 100 * sx) < px.x) {
        x_alpha = clamp(.1 * sx/px.x, 0, max_alpha);
    }
    else if (mod(local_pos.x, 10 * sx) < px.x) {
        x_alpha = clamp(0.01 * sx/px.x, 0, max_alpha);
    }

    float y_alpha = 0.0;
    if (mod(local_pos.y, 1000 * sy) < px.y) {
        y_alpha = clamp(1 * sy/px.y, 0, max_alpha);
    }
    else if (mod(local_pos.y, 100 * sy) < px.y) {
        y_alpha = clamp(.1 * sy/px.y, 0, max_alpha);
    }
    else if (mod(local_pos.y, 10 * sy) < px.y) {
        y_alpha = clamp(0.01 * sy/px.y, 0, max_alpha);
    }

    float alpha = (((log(max(x_alpha, y_alpha))/log(10.))+2) / 3);
    if (alpha == 0) {
        discard;
    }
    gl_FragColor = vec4(1, 1, 1, alpha);
}
"""


class GridLinesVisual(Visual):
    """ Displays regularly spaced grid lines in any coordinate system and at
    any scale.

    Parameters
    ----------
    scale : tuple
        The scale to use.
    """
    def __init__(self, scale=(1, 1), **kwargs):
        super(GridLinesVisual, self).__init__(**kwargs)
        self._program = ModularProgram(VERT, FRAG)
        self._vbo = None
        self._scale = scale
        self._tr_cache = TransformCache()
        self.set_gl_state('additive', cull_face=False)

    def _buffer(self):
        if self._vbo is None:
            # quad covers entire view; frag. shader will deal with image shape
            quad = np.array([[-1, -1], [1, -1], [1, 1],
                             [-1, -1], [1, 1], [-1, 1]],
                            dtype=np.float32)
            self._vbo = gloo.VertexBuffer(quad)
        return self._vbo

    def draw(self, transforms):
        """Draw the visual

        Parameters
        ----------
        transforms : instance of TransformSystem
            The transforms to use.
        """
        Visual.draw(self, transforms)

        doc_to_ndc = self._tr_cache.get([transforms.framebuffer_to_render, 
                                         transforms.document_to_framebuffer])
        self._tr_cache.roll()
        local_to_doc = transforms.visual_to_document

        self._program.frag['map_nd_to_doc'] = doc_to_ndc.inverse
        self._program.frag['map_doc_to_local'] = local_to_doc.inverse
        
        self._program.prepare()
        self._program['pos'] = self._buffer()
        self._program['scale'] = self._scale
        self._program.draw('triangles')



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import weakref

from ..shaders import Function

# To replace color.py soon..


class Alpha(object):
    def __init__(self, alpha=1.0):
        self.shader = Function("""
            void apply_alpha() {
                gl_FragColor.a = gl_FragColor.a * $alpha;
            }
        """)
        self.alpha = alpha
    
    @property
    def alpha(self):
        return self._alpha
    
    @alpha.setter
    def alpha(self, a):
        self._alpha = a
        self.shader['alpha'] = a
        
    def _attach(self, visual):
        self._visual = weakref.ref(visual)
        hook = visual._get_hook('frag', 'post')
        hook.add(self.shader())


class ColorFilter(object):
    def __init__(self, filter=(1, 1, 1, 1)):
        self.shader = Function("""
            void apply_color_filter() {
                gl_FragColor = gl_FragColor * $filter;
            }
        """)
        self.filter = filter
    
    @property
    def filter(self):
        return self._filter
    
    @filter.setter
    def filter(self, f):
        self._filter = tuple(f)
        self.shader['filter'] = self._filter
        
    def _attach(self, visual):
        self._visual = visual
        hook = self._visual._get_hook('frag', 'post')
        hook.add(self.shader())



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from ..util.event import EmitterGroup, Event
from .shaders import StatementList
from .. import gloo

"""
API Issues to work out:

  * Need Visual.bounds() as described here:
    https://github.com/vispy/vispy/issues/141

"""


class Visual(object):
    """
    Abstract class representing a drawable object.

    At a minimum, Visual subclasses should extend the draw() method.

    Events:

        update : Event
            Emitted when the visual has changed and needs to be redrawn.
        bounds_change : Event
            Emitted when the bounds of the visual have changed.

    Notes
    -----
    When used in the scenegraph, all Visual classes are mixed with
    `vispy.scene.Node` in order to implement the methods, attributes and
    capabilities required for their usage within it.
    """

    def __init__(self):
        self._visible = True
        self.events = EmitterGroup(source=self,
                                   auto_connect=True,
                                   update=Event,
                                   bounds_change=Event
                                   )
        self._gl_state = {'preset': None}
        self._filters = set()
        self._hooks = {}

    def set_gl_state(self, preset=None, **kwargs):
        """Define the set of GL state parameters to use when drawing

        Parameters
        ----------
        preset : str
            Preset to use.
        **kwargs : dict
            Keyword argments to use.
        """
        self._gl_state = kwargs
        self._gl_state['preset'] = preset

    def update_gl_state(self, *args, **kwargs):
        """Modify the set of GL state parameters to use when drawing

        Parameters
        ----------
        *args : tuple
            Arguments.
        **kwargs : dict
            Keyword argments.
        """
        if len(args) == 1:
            self._gl_state['preset'] = args[0]
        elif len(args) != 0:
            raise TypeError("Only one positional argument allowed.")
        self._gl_state.update(kwargs)

    def _update(self):
        """
        This method is called internally whenever the Visual needs to be 
        redrawn. By default, it emits the update event.
        """
        self.events.update()

    def draw(self, transforms):
        """Draw this visual now.

        The default implementation calls gloo.set_state().
        
        This function is called automatically when the visual needs to be drawn
        as part of a scenegraph, or when calling 
        ``SceneCanvas.draw_visual(...)``. It is uncommon to call this method 
        manually.
        
        The *transforms* argument is a TransformSystem instance that provides 
        access to transforms that the visual
        may use to determine its relationship to the document coordinate
        system (which provides physical measurements) and the framebuffer
        coordinate system (which is necessary for antialiasing calculations). 
        
        Vertex transformation can be done either on the CPU using 
        Transform.map(), or on the GPU using the GLSL functions generated by 
        Transform.shader_map().

        Parameters
        ----------
        transforms : instance of TransformSystem
            The transforms to use.
        """
        gloo.set_state(**self._gl_state)

    def bounds(self, mode, axis):
        """ Return the (min, max) bounding values describing the location of
        this node in its local coordinate system.

        Parameters
        ----------
        mode : str
            Describes the type of boundary requested. Can be "visual", "data",
            or "mouse".
        axis : 0, 1, 2
            The axis along which to measure the bounding values, in
            x-y-z order.
        
        Returns
        -------
        None or (min, max) tuple. 
        
        Notes
        -----
        This is used primarily to allow automatic ViewBox zoom/pan.
        By default, this method returns None which indicates the object should 
        be ignored for automatic zooming along *axis*.
        
        A scenegraph may also use this information to cull visuals from the
        display list.
        
        """
        return None

    def update(self):
        """
        Emit an event to inform listeners that this Visual needs to be redrawn.
        """
        self.events.update()

    def _get_hook(self, shader, name):
        """Return a FunctionChain that Filters may use to modify the program.

        *shader* should be "frag" or "vert"
        *name* should be "pre" or "post"
        """
        assert name in ('pre', 'post')
        key = (shader, name)
        if key in self._hooks:
            return self._hooks[key]

        prog = getattr(self, '_program', None)
        if prog is None:
            raise NotImplementedError("%s shader does not implement hook '%s'"
                                      % key)
        hook = StatementList()
        if shader == 'vert':
            prog.vert[name] = hook
        elif shader == 'frag':
            prog.frag[name] = hook
        self._hooks[key] = hook
        return hook

    def attach(self, filt):
        """Attach a Filter to this visual

        Each filter modifies the appearance or behavior of the visual.

        Parameters
        ----------
        filt : object
            The filter to attach.
        """
        filt._attach(self)
        self._filters.add(filt)

    def detach(self, filt):
        """Detach a filter

        Parameters
        ----------
        filt : object
            The filter to detach.
        """
        self._filters.remove(filt)
        filt._detach(self)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from .widget import Widget
from ..subscene import SubScene
from ..cameras import make_camera, BaseCamera
from ...ext.six import string_types
from ... import gloo
from ...visuals.components import Clipper
from ...visuals import Visual


class ViewBox(Widget):
    """ Provides a rectangular widget to which its subscene is rendered.

    Three classes work together when using a ViewBox:
    * The :class:`SubScene` class describes a "world" coordinate system and the
    entities that live inside it.
    * ViewBox is a "window" through which we view the
    subscene. Multiple ViewBoxes may view the same subscene.
    * :class:`Camera` describes both the perspective from which the
    subscene is rendered, and the way user interaction affects that
    perspective.

    In general it is only necessary to create the ViewBox; a SubScene and
    Camera will be generated automatically.

    Parameters
    ----------
    camera : None, :class:`Camera`, or str
        The camera through which to view the SubScene. If None, then a
        PanZoomCamera (2D interaction) is used. If str, then the string is
        used as the argument to :func:`make_camera`.
    scene : None or :class:`SubScene`
        The `SubScene` instance to view. If None, a new `SubScene` is created.
    clip_method : str
        Clipping method to use.
    **kwargs : dict
        Extra keyword arguments to pass to `Widget`.
    """
    def __init__(self, camera=None, scene=None, clip_method='fragment',
                 **kwargs):

        self._camera = None
        Widget.__init__(self, **kwargs)

        # Init method to provide a pixel grid
        self._clip_method = clip_method
        self._clipper = Clipper()

        # Each viewbox has a scene widget, which has a transform that
        # represents the transformation imposed by camera.
        if scene is None:
            if self.name is not None:
                name = str(self.name) + "_Scene"
            else:
                name = None
            self._scene = SubScene(name=name)
        elif isinstance(scene, SubScene):
            self._scene = scene
        else:
            raise TypeError('Argument "scene" must be None or SubScene.')
        self._scene.add_parent(self)

        # Camera is a helper object that handles scene transformation
        # and user interaction.
        if camera is None:
            camera = 'base'
        if isinstance(camera, string_types):
            self.camera = make_camera(camera, parent=self.scene)
        elif isinstance(camera, BaseCamera):
            self.camera = camera
        else:
            raise TypeError('Argument "camera" must be None, str, or Camera.')

    @property
    def camera(self):
        """ Get/set the Camera in use by this ViewBox

        If a string is given (e.g. 'panzoom', 'turntable', 'fly'). A
        corresponding camera is selected if it already exists in the
        scene, otherwise a new camera is created.

        The camera object is made a child of the scene (if it is not
        already in the scene).

        Multiple cameras can exist in one scene, although only one can
        be active at a time. A single camera can be used by multiple
        viewboxes at the same time.
        """
        return self._camera

    @camera.setter
    def camera(self, cam):
        if isinstance(cam, string_types):
            # Try to select an existing camera
            for child in self.scene.children:
                if isinstance(child, BaseCamera):
                    this_cam_type = child.__class__.__name__.lower()[:-6]
                    if this_cam_type == cam:
                        self.camera = child
                        return
            else:
                # No such camera yet, create it then
                self.camera = make_camera(cam)
            
        elif isinstance(cam, BaseCamera):
            # Ensure that the camera is in the scene
            if not self.is_in_scene(cam):
                cam.add_parent(self.scene)
            # Disconnect / connect
            if self._camera is not None:
                self._camera._viewbox_unset(self)
            self._camera = cam
            if self._camera is not None:
                self._camera._viewbox_set(self)
            # Update view
            cam.view_changed()
        
        else:
            raise ValueError('Not a camera object.')

    def is_in_scene(self, node):
        """Get whether the given node is inside the scene of this viewbox.

        Parameters
        ----------
        node : instance of Node
            The node.
        """
        def _is_child(parent, child):
            if child in parent.children:
                return True
            else:
                for c in parent.children:
                    if isinstance(c, ViewBox):
                        continue
                    elif _is_child(c, child):
                        return True
            return False
        
        return _is_child(self.scene, node)
    
    def get_scene_bounds(self, dim=None):
        """Get the total bounds based on the visuals present in the scene

        Parameters
        ----------
        dim : int | None
            Dimension to return.

        Returns
        -------
        bounds : list | tuple
            If ``dim is None``, Returns a list of 3 tuples, otherwise
            the bounds for the requested dimension.
        """
        # todo: handle sub-children
        # todo: handle transformations
        # Init
        mode = 'data'  # or visual?
        bounds = [(np.inf, -np.inf), (np.inf, -np.inf), (np.inf, -np.inf)]
        # Get bounds of all children
        for ob in self.scene.children:
            if hasattr(ob, 'bounds'):
                for axis in (0, 1, 2):
                    if (dim is not None) and dim != axis:
                        continue
                    b = ob.bounds(mode, axis)
                    if b is not None:
                        b = min(b), max(b)  # Ensure correct order
                        bounds[axis] = (min(bounds[axis][0], b[0]), 
                                        max(bounds[axis][1], b[1]))
        # Set defaults
        for axis in (0, 1, 2):
            if any(np.isinf(bounds[axis])):
                bounds[axis] = -1, 1
        
        if dim is not None:
            return bounds[dim]
        else:
            return bounds
    
    @property
    def scene(self):
        """ The root node of the scene viewed by this ViewBox.
        """
        return self._scene

    def add(self, node):
        """ Add an Node to the scene for this ViewBox.

        This is a convenience method equivalent to
        `node.add_parent(viewbox.scene)`

        Parameters
        ----------
        node : instance of Node
            The node to add.
        """
        node.add_parent(self.scene)

    @property
    def clip_method(self):
        """ The method used to clip the subscene to the boundaries of the 
        viewbox.

        Clipping methods are:
        
        * None - do not perform clipping. The default for now.
        * 'viewport' - use glViewPort to provide a clipped sub-grid
          onto the parent pixel grid, if possible.
        * 'fbo' - use an FBO to draw the subscene to a texture, and
          then render the texture in the parent scene.
        * 'fragment' - clipping in the fragment shader
        * 'stencil' - TODO

        Notes
        -----
        The 'viewport' method requires that the transformation (from
        the pixel grid to this viewbox) is translate+scale only. If
        this is not the case, the method falls back to the default.

        The 'fbo' method is convenient when the result of the viewbox
        should be reused. Otherwise the overhead can be significant and
        the image can get slightly blurry if the transformations do
        not match.

        It is possible to have a graph with multiple stacked viewboxes
        which each use different methods (subject to the above
        restrictions).

        """
        return self._clip_method

    @clip_method.setter
    def clip_method(self, value):
        valid_methods = (None, 'fragment', 'viewport', 'fbo')
        if value not in valid_methods:
            t = 'clip_method should be in %s' % str(valid_methods)
            raise ValueError((t + ', not %r') % value)
        self._clip_method = value
        self.update()

    def draw(self, event):
        """ Draw the viewbox border/background

        This also prepares to draw the
        subscene using the configured clipping method.

        Parameters
        ----------
        event : instance of Event
            The draw event.
        """
        # -- Calculate resolution

        # Get current transform and calculate the 'scale' of the viewbox
        size = self.size
        transform = event.visual_to_document
        p0, p1 = transform.map((0, 0)), transform.map(size)
        res = (p1 - p0)[:2]
        res = abs(res[0]), abs(res[1])

        # Set resolution (note that resolution can be non-integer)
        self._resolution = res

        method = self.clip_method
        viewport, fbo = None, None

        if method == 'fragment':
            self._prepare_fragment()
        elif method == 'stencil':
            raise NotImplementedError('No stencil buffer clipping yet.')
        elif method == 'viewport':
            viewport = self._prepare_viewport(event)
        elif method == 'fbo':
            fbo = self._prepare_fbo(event)
        else:
            raise ValueError('Unknown clipping method %s' % method)

        # -- Draw
        super(ViewBox, self).draw(event)

        event.push_viewbox(self)

        # make sure the current drawing system does not attempt to draw
        # the scene.
        event.handled_children.append(self.scene)
        if fbo:
            canvas_transform = event.visual_to_canvas
            offset = canvas_transform.map((0, 0))[:2]
            size = canvas_transform.map(self.size)[:2] - offset

            # Ask the canvas to activate the new FBO
            event.push_fbo(fbo, offset, size)
            event.push_node(self.scene)
            try:
                # Draw subscene to FBO
                self.scene.draw(event)
            finally:
                event.pop_node()
                event.pop_fbo()

            gloo.set_state(cull_face=False)
        elif viewport:
            # Push viewport, draw, pop it
            event.push_viewport(viewport)
            event.push_node(self.scene)
            try:
                self.scene.draw(event)
            finally:
                event.pop_node()
                event.pop_viewport()
        elif method == 'fragment':
            self._clipper.bounds = event.visual_to_framebuffer.map(self.rect)
            event.push_node(self.scene)
            try:
                self.scene.draw(event)
            finally:
                event.pop_node()

        event.pop_viewbox()

    def _prepare_fragment(self, root=None):
        # Todo: should only be run when there are changes in the graph, not
        # on every frame.
        if root is None:
            root = self.scene

        for ch in root.children:
            if isinstance(ch, Visual):
                try:
                    ch.attach(self._clipper)
                except NotImplementedError:
                    # visual does not support clipping
                    pass
            self._prepare_fragment(ch)

    def _prepare_viewport(self, event):
        fb_transform = event.node_transform(map_from=event.node_cs,
                                            map_to=event.framebuffer_cs)
        p1 = fb_transform.map((0, 0))
        p2 = fb_transform.map(self.size)
        return p1[0], p1[1], p2[0]-p1[0], p2[1]-p1[1]

    def _prepare_fbo(self, event):
        """ Draw the viewbox via an FBO. This method can be applied
        in any situation, regardless of the transformations to this
        viewbox.

        TODO:
        Right now, this implementation create a program, texture and FBO
        on *each* draw, because it does not work otherwise. This is probably
        a bug in gloo that prevents using two FBO's / programs at the same
        time.

        Also, we use plain gloo and calculate the transformation
        ourselves, assuming 2D only. Eventually we should just use the
        transform of self. I could not get that to work, probably
        because I do not understand the component system yet.
        """

        render_vertex = """
            attribute vec3 a_position;
            attribute vec2 a_texcoord;
            varying vec2 v_texcoord;
            void main()
            {
                gl_Position = vec4(a_position, 1.0);
                v_texcoord = a_texcoord;
            }
        """

        render_fragment = """
            uniform sampler2D u_texture;
            varying vec2 v_texcoord;
            void main()
            {
                vec4 v = texture2D(u_texture, v_texcoord);
                gl_FragColor = vec4(v.rgb, 1.0);
            }
        """

        # todo: don't do this on every draw
        if True:
            # Create program
            self._fboprogram = gloo.Program(render_vertex, render_fragment)
            # Create texture
            self._tex = gloo.Texture2D((10, 10, 4), interpolation='linear')
            self._fboprogram['u_texture'] = self._tex
            # Create texcoords and vertices
            # Note y-axis is inverted here because the viewbox coordinate
            # system has origin in the upper-left, but the image is rendered
            # to the framebuffer with origin in the lower-left.
            texcoord = np.array([[0, 1], [1, 1], [0, 0], [1, 0]],
                                dtype=np.float32)
            position = np.zeros((4, 3), np.float32)
            self._fboprogram['a_texcoord'] = gloo.VertexBuffer(texcoord)
            self._fboprogram['a_position'] = self._vert = \
                gloo.VertexBuffer(position)

        # Get fbo, ensure it exists
        fbo = getattr(self, '_fbo', None)
        if True:  # fbo is None:
            self._fbo = 4
            self._fbo = fbo = gloo.FrameBuffer(self._tex,
                                               gloo.RenderBuffer((10, 10)))

        # Set texture coords to make the texture be drawn in the right place
        # Note that we would just use -1..1 if we would use a Visual.
        coords = [[0, 0], [self.size[0], self.size[1]]]
        
        transform = event.get_full_transform()
        coords = transform.map(coords)
        x1, y1, z = coords[0][:3]
        x2, y2, z = coords[1][:3]
        vertices = np.array([[x1, y1, z], [x2, y1, z],
                             [x1, y2, z], [x2, y2, z]],
                            np.float32)
        self._vert.set_data(vertices)

        # Set fbo size (mind that this is set using shape!)
        resolution = [int(i+0.5) for i in self._resolution]  # set in draw()
        shape = resolution[1], resolution[0]
        # todo: use fbo.resize(shape)
        fbo.color_buffer.resize(shape+(4,))
        fbo.depth_buffer.resize(shape)

        return fbo



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from .mesh import MeshVisual
from ..geometry.isosurface import isosurface


class IsosurfaceVisual(MeshVisual):
    """Displays an isosurface of a 3D scalar array.

    Parameters
    ----------
    data : ndarray | None
        3D scalar array.
    level: float | None
        The level at which the isosurface is constructed from *data*.

    Notes
    -----
    """
    def __init__(self, data=None, level=None, **kwargs):
        self._data = None
        self._level = level
        self._recompute = True
        MeshVisual.__init__(self, **kwargs)
        if data is not None:
            self.set_data(data)

    @property
    def level(self):
        """ The threshold at which the isosurface is constructed from the
        3D data.
        """
        return self._level

    @level.setter
    def level(self, level):
        self._level = level
        self._recompute = True
        self.update()

    def set_data(self, data):
        """ Set the scalar array data

        Parameters
        ----------
        data : ndarray
            A 3D array of scalar values. The isosurface is constructed to show
            all locations in the scalar field equal to ``self.level``.
        """
        self._data = data
        self._recompute = True
        self.update()

    def draw(self, transforms):
        """Draw the visual

        Parameters
        ----------
        transforms : instance of TransformSystem
            The transforms to use.
        """
        if self._data is None or self._level is None:
            return

        if self._recompute:
            verts, faces = isosurface(self._data, self._level)
            MeshVisual.set_data(self, vertices=verts, faces=faces)
            self._recompute = False

        MeshVisual.draw(self, transforms)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import weakref

from .. import gloo
from .. import app
from .node import Node
from ..visuals.transforms import STTransform, TransformCache
from ..color import Color
from ..util import logger
from ..util.profiler import Profiler
from .subscene import SubScene
from .events import SceneDrawEvent, SceneMouseEvent
from .widgets import Widget


class SceneCanvas(app.Canvas):
    """A Canvas that automatically draws the contents of a scene

    Parameters
    ----------
    title : str
        The widget title
    size : (width, height)
        The size of the window.
    position : (x, y)
        The position of the window in screen coordinates.
    show : bool
        Whether to show the widget immediately. Default False.
    autoswap : bool
        Whether to swap the buffers automatically after a draw event.
        Default True. If True, the ``swap_buffers`` Canvas method will
        be called last (by default) by the ``canvas.draw`` event handler.
    app : Application | str
        Give vispy Application instance to use as a backend.
        (vispy.app is used by default.) If str, then an application
        using the chosen backend (e.g., 'pyglet') will be created.
        Note the canvas application can be accessed at ``canvas.app``.
    create_native : bool
        Whether to create the widget immediately. Default True.
    vsync : bool
        Enable vertical synchronization.
    resizable : bool
        Allow the window to be resized.
    decorate : bool
        Decorate the window. Default True.
    fullscreen : bool | int
        If False, windowed mode is used (default). If True, the default
        monitor is used. If int, the given monitor number is used.
    config : dict
        A dict with OpenGL configuration options, which is combined
        with the default configuration options and used to initialize
        the context. See ``canvas.context.config`` for possible
        options.
    shared : Canvas | GLContext | None
        An existing canvas or context to share OpenGL objects with.
    keys : str | dict | None
        Default key mapping to use. If 'interactive', escape and F11 will
        close the canvas and toggle full-screen mode, respectively.
        If dict, maps keys to functions. If dict values are strings,
        they are assumed to be ``Canvas`` methods, otherwise they should
        be callable.
    parent : widget-object
        The parent widget if this makes sense for the used backend.
    dpi : float | None
        Resolution in dots-per-inch to use for the canvas. If dpi is None,
        then the value will be determined by querying the global config first,
        and then the operating system.
    always_on_top : bool
        If True, try to create the window in always-on-top mode.
    px_scale : int > 0
        A scale factor to apply between logical and physical pixels in addition
        to the actual scale factor determined by the backend. This option
        allows the scale factor to be adjusted for testing.
    bgcolor : Color
        The background color to use.

    See also
    --------
    vispy.app.Canvas

    Notes
    -----
    Receives the following events:

        * initialize
        * resize
        * draw
        * mouse_press
        * mouse_release
        * mouse_double_click
        * mouse_move
        * mouse_wheel
        * key_press
        * key_release
        * stylus
        * touch
        * close

    The ordering of the mouse_double_click, mouse_press, and mouse_release
    events are not guaranteed to be consistent between backends. Only certain
    backends natively support double-clicking (currently Qt and WX); on other
    backends, they are detected manually with a fixed time delay.
    This can cause problems with accessibility, as increasing the OS detection
    time or using a dedicated double-click button will not be respected.
    """
    def __init__(self, title='Vispy canvas', size=(800, 600), position=None,
                 show=False, autoswap=True, app=None, create_native=True,
                 vsync=False, resizable=True, decorate=True, fullscreen=False,
                 config=None, shared=None, keys=None, parent=None, dpi=None,
                 always_on_top=False, px_scale=1, bgcolor='black'):
        self._fb_stack = []  # for storing information about framebuffers used
        self._vp_stack = []  # for storing information about viewports used
        self._scene = None
        
        # A default widget that follows the shape of the canvas
        self._central_widget = None

        self._bgcolor = Color(bgcolor).rgba

        super(SceneCanvas, self).__init__(
            title, size, position, show, autoswap, app, create_native, vsync,
            resizable, decorate, fullscreen, config, shared, keys, parent, dpi,
            always_on_top, px_scale)
        self.events.mouse_press.connect(self._process_mouse_event)
        self.events.mouse_move.connect(self._process_mouse_event)
        self.events.mouse_release.connect(self._process_mouse_event)
        self.events.mouse_wheel.connect(self._process_mouse_event)

        # Collection of transform caches; one for each root visual used in 
        # self.draw_visual(...)
        self._transform_caches = weakref.WeakKeyDictionary()

        # Set up default node stack: ndc -> fb -> canvas -> scene
        self.render_cs = Node(name="render_cs")
        self.framebuffer_cs = Node(parent=self.render_cs, 
                                   name="framebuffer_cs")
        self.framebuffer_cs.transform = STTransform()
        self.canvas_cs = Node(parent=self.framebuffer_cs,
                              name="canvas_cs")
        self.canvas_cs.transform = STTransform()
        # By default, the document coordinate system is the canvas.
        self.canvas_cs.document = self.canvas_cs
        
        self.scene = SubScene(parent=self.canvas_cs)
        
    @property
    def scene(self):
        """ The SubScene object that represents the root node of the
        scene graph to be displayed.
        """
        return self._scene

    @scene.setter
    def scene(self, e):
        if self._scene is not None:
            self._scene.events.update.disconnect(self._scene_update)
        self._scene = e
        self._scene.events.update.connect(self._scene_update)

    @property
    def central_widget(self):
        """ Returns the default widget that occupies the entire area of the
        canvas. 
        """
        if self._central_widget is None:
            self._central_widget = Widget(size=self.size, parent=self.scene)
        return self._central_widget

    def _scene_update(self, event):
        self.update()

    def on_draw(self, event):
        """Draw handler

        Parameters
        ----------
        event : instance of Event
            The draw event.
        """
        if self._scene is None:
            return  # Can happen on initialization
        logger.debug('Canvas draw')

        self._draw_scene()

    def render(self, region=None, size=None):
        """ Render the scene to an offscreen buffer and return the image array.
        
        Parameters
        ----------
        region : tuple | None
            Specifies the region of the canvas to render. Format is 
            (x, y, w, h). By default, the entire canvas is rendered.
        size : tuple | None
            Specifies the size of the image array to return. If no size is 
            given, then the size of the *region* is used. This argument allows
            the scene to be rendered at resolutions different from the native
            canvas resolution.

        Returns
        -------
        image : array
            Numpy array of type ubyte and shape (h, w, 4). Index [0, 0] is the 
            upper-left corner of the rendered region.
        
        """
        # Set up a framebuffer to render to
        offset = (0, 0) if region is None else region[:2]
        csize = self.size if region is None else region[2:]
        size = csize if size is None else size
        fbo = gloo.FrameBuffer(color=gloo.RenderBuffer(size[::-1]),
                               depth=gloo.RenderBuffer(size[::-1]))

        self.push_fbo(fbo, offset, csize)
        try:
            self._draw_scene(viewport=(0, 0) + size)
            return fbo.read()
        finally:
            self.pop_fbo()

    def _draw_scene(self, viewport=None):
        self.context.clear(color=self._bgcolor, depth=True)
        # Draw the scene, but first disconnect its change signal--
        # any changes that take place during the paint should not trigger
        # a subsequent repaint.
        with self.scene.events.update.blocker(self._scene_update):
            self.draw_visual(self.scene, viewport=viewport)

    def draw_visual(self, visual, event=None, viewport=None):
        """ Draw a visual to the canvas or currently active framebuffer.
        
        Parameters
        ----------
        visual : Visual
            The visual to draw
        event : None or DrawEvent
            Optionally specifies the original canvas draw event that initiated
            this draw.
        viewport : tuple | None
            Optionally specifies the viewport to use. If None, the entire
            physical size is used.
        """
        self.set_current()
        prof = Profiler()
        nfb = len(self._fb_stack)
        nvp = len(self._vp_stack)
        
        # Create draw event, which keeps track of the path of transforms
        self._process_node_count = 0  # for debugging
        
        # Get the cache of transforms used for this visual
        tr_cache = self._transform_caches.setdefault(visual, TransformCache())
        # and mark the entire cache as aged
        tr_cache.roll()
        prof('roll transform cache')
        
        scene_event = SceneDrawEvent(canvas=self, event=event, 
                                     transform_cache=tr_cache)
        prof('create SceneDrawEvent')
        
        vp = (0, 0) + self.physical_size if viewport is None else viewport
        scene_event.push_viewport(vp)
        prof('push_viewport')
        try:
            # Force update of transforms on base entities
            # TODO: this should happen as a reaction to resize, push_viewport,
            #       etc.; not here.  (but note the transforms must change
            #       following push_viewport)
            self.fb_ndc_transform
            self.canvas_fb_transform
            
            scene_event.push_node(self.render_cs)
            scene_event.push_node(self.framebuffer_cs)
            scene_event.push_node(self.canvas_cs)
            scene_event.push_node(visual)
            prof('initialize event scenegraph')
            visual.draw(scene_event)
            prof('draw scene')
        finally:
            scene_event.pop_viewport()

        if len(self._vp_stack) > nvp:
            logger.warning("Viewport stack not fully cleared after draw.")
        if len(self._fb_stack) > nfb:
            logger.warning("Framebuffer stack not fully cleared after draw.")

    def _process_mouse_event(self, event):
        prof = Profiler()
        tr_cache = self._transform_caches.setdefault(self.scene, 
                                                     TransformCache())
        scene_event = SceneMouseEvent(canvas=self, event=event,
                                      transform_cache=tr_cache)
        scene_event.push_node(self.render_cs)
        scene_event.push_node(self.framebuffer_cs)
        scene_event.push_node(self.canvas_cs)
        scene_event.push_node(self._scene)
        prof('prepare mouse event')
        
        self._scene._process_mouse_event(scene_event)
        prof('process')
        
        # If something in the scene handled the scene_event, then we mark
        # the original event accordingly.
        event.handled = scene_event.handled

    def on_resize(self, event):
        """Resize handler

        Parameters
        ----------
        event : instance of Event
            The resize event.
        """
        if self._central_widget is not None:
            self._central_widget.size = self.size

    # -------------------------------------------------- transform handling ---
    def push_viewport(self, viewport):
        """ Push a viewport on the stack

        It is the responsibility of the caller to ensure the given values are
        int. The viewport's origin is defined relative to the current
        viewport.

        Parameters
        ----------
        viewport : tuple
            The viewport as (x, y, w, h).
        """
        vp = list(viewport)
        # Normalize viewport before setting;
        if vp[2] < 0:
            vp[0] += vp[2]
            vp[2] *= -1
        if vp[3] < 0:
            vp[1] += vp[3]
            vp[3] *= -1

        self._vp_stack.append(vp)
        self.fb_ndc_transform  # update!
        # Apply
        try:
            self._set_viewport(vp)
        except:
            self._vp_stack.pop()
            self.fb_ndc_transform  # update!
            raise

    def pop_viewport(self):
        """ Pop a viewport from the stack.
        """
        vp = self._vp_stack.pop()
        # Activate latest
        if len(self._vp_stack) > 0:
            self._set_viewport(self._vp_stack[-1])
            self.fb_ndc_transform  # update!
        return vp

    def _set_viewport(self, vp):
        self.context.set_viewport(*vp)

    def push_fbo(self, fbo, offset, csize):
        """ Push an FBO on the stack, together with the new viewport.
        and the transform to the FBO.

        Parameters
        ----------
        fbo : instance of FrameBuffer
            The framebuffer.
        offset : tuple
            The offset.
        csize : tuple
            The size to use.
        """
        self._fb_stack.append((fbo, offset, csize))
        self.canvas_fb_transform  # update!

        # Apply
        try:
            fbo.activate()
            h, w = fbo.color_buffer.shape[:2]
            self.push_viewport((0, 0, w, h))
        except Exception:
            self._fb_stack.pop()
            raise

    def pop_fbo(self):
        """ Pop an FBO from the stack.
        """
        fbo = self._fb_stack.pop()
        fbo[0].deactivate()
        self.pop_viewport()
        if len(self._fb_stack) > 0:
            old_fbo = self._fb_stack[-1]
            old_fbo[0].activate()
        self.canvas_fb_transform  # update!
        return fbo
        
    def _current_framebuffer(self):
        """ Return (fbo, origin, canvas_size) for the current
        FBO on the stack, or for the canvas if there is no FBO.
        """
        if len(self._fb_stack) == 0:
            return None, (0, 0), self.size
        else:
            return self._fb_stack[-1]

    @property
    def canvas_fb_transform(self):
        """ The transform that maps from the canvas coordinate system to the
        current framebuffer coordinate system. 
        
        The framebuffer coordinate 
        system is used for antialiasing calculations, and is also the 
        system used when specifying coordinates for glViewport 
        (or gloo.set_viewport). Its origin is in the lower-left corner (as
        opposed to the document / canvas coordinate system, which has its
        origin in the upper-left corner).
        
        Often the canvas and framebuffer coordinate systems are identical. 
        However, some systems with high-resolution 
        displays may use framebuffers with higher resolution than the reported
        size of the canvas. Likewise, when rendering to an FBO, the resolution
        and offset of the framebuffer may not match the canvas. 
        """
        fbo, offset, csize = self._current_framebuffer()
        if fbo is None:
            fbsize = self.physical_size
        else:
            fbsize = fbo.color_buffer.shape
            # image shape is (rows, cols), unlike canvas shape.
            fbsize = fbsize[1], fbsize[0]  

        map_from = [list(offset), [offset[0] + csize[0], offset[1] + csize[1]]]
        map_to = [[0, fbsize[1]], [fbsize[0], 0]]
        
        self.canvas_cs.transform.set_mapping(map_from, map_to)
        return self.canvas_cs.transform

    @property
    def fb_ndc_transform(self):
        """ The transform that maps from the framebuffer coordinate system to
        normalized device coordinates (which is the obligatory output 
        coordinate system for all vertex shaders). This transform accounts for
        the current glViewport.
        """
        offset, csize, fbsize = self._current_framebuffer()
        x, y, w, h = self._vp_stack[-1]
        
        map_from = [[x, y], [x+w, y+h]]
        map_to = [[-1, -1], [1, 1]]
        
        self.framebuffer_cs.transform.set_mapping(map_from, map_to)
        return self.framebuffer_cs.transform
    
    @property
    def render_transform(self):
        """ The transform that maps from the Canvas pixel coordinate system 
        <(0, 0) at top-left, (w, h) at bottom-right> to normalized device 
        coordinates within the current glViewport and FBO.

        Most visuals should use this transform when drawing.
        """
        return self.fb_ndc_transform * self.canvas_fb_transform

    @property
    def bgcolor(self):
        return Color(self._bgcolor)

    @bgcolor.setter
    def bgcolor(self, color):
        self._bgcolor = Color(color).rgba
        if hasattr(self, '_backend'):
            self.update()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from ..node import Node
from ...visuals.mesh import MeshVisual
from ...visuals.transforms import STTransform
from ...util.event import Event
from ...geometry import Rect
from ...color import Color


class Widget(Node):
    """ A widget takes up a rectangular space, intended for use in
    a 2D pixel coordinate frame.

    The widget is positioned using the transform attribute (as any
    node), and its extent (size) is kept as a separate property.

    Parameters
    ----------
    pos : (x, y)
        A 2-element tuple to specify the top left corner of the widget.
    size : (w, h)
        A 2-element tuple to spicify the size of the widget.
    border_color : color
        The color of the border.
    bgcolor : color
        The background color.
    clip : bool
        Not used :)
    padding : int
        The amount of padding in the widget (i.e. the space reserved between
        the contents and the border).
    margin : int
        The margin to keep outside the widget's border.
    """

    def __init__(self, pos=(0, 0), size=(10, 10), border_color=None,
                 bgcolor=None, clip=False, padding=0, margin=0, **kwargs):
        Node.__init__(self, **kwargs)

        # For drawing border.
        # A mesh is required because GL lines cannot be drawn with predictable
        # shape across all platforms.
        self._border_color = self._bgcolor = Color(None)
        self._face_colors = None
        self._visual = MeshVisual(mode='triangles')
        self._visual.set_gl_state('translucent', depth_test=False)

        # whether this widget should clip its children
        # (todo)
        self._clip = clip

        # reserved space inside border
        self._padding = padding

        # reserved space outside border
        self._margin = margin

        self.events.add(resize=Event)
        self._size = 16, 16
        self.transform = STTransform()
        # todo: TTransform (translate only for widgets)

        self._widgets = []
        self.pos = pos
        self.size = size
        self.border_color = border_color
        self.bgcolor = bgcolor

    @property
    def pos(self):
        return tuple(self.transform.translate[:2])

    @pos.setter
    def pos(self, p):
        assert isinstance(p, tuple)
        assert len(p) == 2
        if p == self.pos:
            return
        self.transform.translate = p[0], p[1], 0, 0
        self._update_line()
        #self.events.resize()

    @property
    def size(self):
        # Note that we cannot let the size be reflected in the transform.
        # Consider a widget of 40x40 in a pixel grid, a child widget therin
        # with size 20x20 would get a scale of 800x800!
        return self._size

    @size.setter
    def size(self, s):
        assert isinstance(s, tuple)
        assert len(s) == 2
        if self._size == s:
            return
        self._size = s
        self._update_line()
        self.events.resize()
        self._update_child_widgets()

    @property
    def rect(self):
        return Rect((0, 0), self.size)

    @rect.setter
    def rect(self, r):
        with self.events.resize.blocker():
            self.pos = r.pos
            self.size = r.size
        self.update()
        self.events.resize()

    @property
    def inner_rect(self):
        """The rectangular area inside the margin, border and padding.

        Generally widgets should avoid drawing or placing widgets outside this
        rectangle.
        """
        m = self.margin + self.padding
        if not self.border_color.is_blank:
            m += 1
        return Rect((m, m), (self.size[0]-2*m, self.size[1]-2*m))

    @property
    def border_color(self):
        """ The color of the border.
        """
        return self._border_color

    @border_color.setter
    def border_color(self, b):
        self._border_color = Color(b)
        self._update_colors()
        self._update_line()
        self.update()

    @property
    def bgcolor(self):
        """ The background color of the Widget.
        """
        return self._bgcolor

    @bgcolor.setter
    def bgcolor(self, value):
        self._bgcolor = Color(value)
        self._update_colors()
        self._update_line()
        self.update()

    @property
    def margin(self):
        return self._margin

    @margin.setter
    def margin(self, m):
        self._margin = m
        self._update_line()

    @property
    def padding(self):
        return self._padding

    @padding.setter
    def padding(self, p):
        self._padding = p
        self._update_child_widgets()

    def _update_line(self):
        """ Update border line to match new shape """
        w = 1  # XXX Eventually this can be a parameter
        m = int(self.margin)
        # border is drawn within the boundaries of the widget:
        #
        #  size = (8, 7)  margin=2
        #  internal rect = (3, 3, 2, 1)
        #  ........
        #  ........
        #  ..BBBB..
        #  ..B  B..
        #  ..BBBB..
        #  ........
        #  ........
        #
        l = b = m
        r = int(self.size[0]) - m
        t = int(self.size[1]) - m
        pos = np.array([
            [l, b], [l+w, b+w],
            [r, b], [r-w, b+w],
            [r, t], [r-w, t-w],
            [l, t], [l+w, t-w],
        ], dtype=np.float32)
        faces = np.array([
            [0, 2, 1],
            [1, 2, 3],
            [2, 4, 3],
            [3, 5, 4],
            [4, 5, 6],
            [5, 7, 6],
            [6, 0, 7],
            [7, 0, 1],
            [5, 3, 1],
            [1, 5, 7],
        ], dtype=np.int32)
        start = 8 if self._border_color.is_blank else 0
        stop = 8 if self._bgcolor.is_blank else 10
        face_colors = None
        if self._face_colors is not None:
            face_colors = self._face_colors[start:stop]
        self._visual.set_data(vertices=pos, faces=faces[start:stop],
                              face_colors=face_colors)

    def _update_colors(self):
        self._face_colors = np.concatenate(
            (np.tile(self.border_color.rgba, (8, 1)),
             np.tile(self.bgcolor.rgba, (2, 1)))).astype(np.float32)

    def draw(self, event):
        """Draw the widget borders

        Parameters
        ----------
        event : instance of Event
            The event containing the transforms.
        """
        if self.border_color.is_blank and self.bgcolor.is_blank:
            return
        self._visual.draw(event)

    def on_resize(self, event):
        """On resize handler

        Parameters
        ----------
        event : instance of Event
            The resize event.
        """
        self._update_child_widgets()

    def _update_child_widgets(self):
        # Set the position and size of child boxes (only those added
        # using add_widget)
        for ch in self._widgets:
            ch.rect = self.rect.padded(self.padding + self.margin)

    def add_widget(self, widget):
        """
        Add a Widget as a managed child of this Widget.

        The child will be
        automatically positioned and sized to fill the entire space inside
        this Widget (unless _update_child_widgets is redefined).

        Parameters
        ----------
        widget : instance of Widget
            The widget to add.

        Returns
        -------
        widget : instance of Widget
            The widget.
        """
        self._widgets.append(widget)
        widget.parent = self
        self._update_child_widgets()
        return widget

    def add_grid(self, *args, **kwargs):
        """
        Create a new Grid and add it as a child widget.

        All arguments are given to Grid().
        """
        from .grid import Grid
        grid = Grid(*args, **kwargs)
        return self.add_widget(grid)

    def add_view(self, *args, **kwargs):
        """
        Create a new ViewBox and add it as a child widget.

        All arguments are given to ViewBox().
        """
        from .viewbox import ViewBox
        view = ViewBox(*args, **kwargs)
        return self.add_widget(view)

    def remove_widget(self, widget):
        """
        Remove a Widget as a managed child of this Widget.

        Parameters
        ----------
        widget : instance of Widget
            The widget to remove.
        """
        self._widgets.remove(widget)
        widget.remove_parent(self)
        self._update_child_widgets()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.


"""
Simple ellipse visual based on PolygonVisual
"""

from __future__ import division

import numpy as np
from ..color import Color
from .polygon import PolygonVisual


class EllipseVisual(PolygonVisual):
    """
    Displays a 2D ellipse

    Parameters
    ----------
    pos : array
        Center of the ellipse
    color : instance of Color
        The face color to use.
    border_color : instance of Color
        The border color to use.
    radius : float | tuple
        Radius or radii of the ellipse
        Defaults to  (0.1, 0.1)
    start_angle : float
        Start angle of the ellipse in degrees
        Defaults to 0.
    span_angle : float
        Span angle of the ellipse in degrees
        Defaults to 0.
    num_segments : int
        Number of segments to be used to draw the ellipse
        Defaults to 100
    """
    def __init__(self, pos=None, color='black', border_color=None,
                 radius=(0.1, 0.1), start_angle=0., span_angle=360.,
                 num_segments=100, **kwargs):
        super(EllipseVisual, self).__init__()
        self.mesh.mode = 'triangle_fan'
        self._vertices = None
        self._pos = pos
        self._color = Color(color)
        self._border_color = Color(border_color)
        self._radius = radius
        self._start_angle = start_angle
        self._span_angle = span_angle
        self._num_segments = num_segments
        self._update()

    def _generate_vertices(self, pos, radius, start_angle, span_angle,
                           num_segments):
        if isinstance(radius, (list, tuple)):
            if len(radius) == 2:
                xr, yr = radius
            else:
                raise ValueError("radius must be float or 2 value tuple/list"
                                 " (got %s of length %d)" % (type(radius),
                                                             len(radius)))
        else:
            xr = yr = radius
        curve_segments = int(num_segments * span_angle / 360.)
        start_angle *= (np.pi/180.)
        self._vertices = np.empty([curve_segments+2, 2], dtype=np.float32)
        self._vertices[0] = np.float32([pos[0], pos[1]])
        theta = np.linspace(start_angle, start_angle + (span_angle/180.)*np.pi,
                            curve_segments+1)
        self._vertices[1:, 0] = pos[0] + xr * np.cos(theta)
        self._vertices[1:, 1] = pos[1] + yr * np.sin(theta)

    @property
    def radius(self):
        """ The start radii of the ellipse.
        """
        return self._radius

    @radius.setter
    def radius(self, radius):
        self._radius = radius
        self._update()

    @property
    def start_angle(self):
        """ The start start_angle of the ellipse.
        """
        return self._start_angle

    @start_angle.setter
    def start_angle(self, start_angle):
        self._start_angle = start_angle
        self._update()

    @property
    def span_angle(self):
        """ The angular span of the ellipse.
        """
        return self._span_angle

    @span_angle.setter
    def span_angle(self, span_angle):
        self._span_angle = span_angle
        self._update()

    @property
    def num_segments(self):
        """ The number of segments in the ellipse.
        """
        return self._num_segments

    @num_segments.setter
    def num_segments(self, num_segments):
        if num_segments < 1:
            raise ValueError('EllipseVisual must consist of more than 1 '
                             'segment')
        self._num_segments = num_segments
        self._update()

    def _update(self):
        if self._pos is None:
            return
        
        self._generate_vertices(pos=self._pos, radius=self._radius,
                                start_angle=self._start_angle,
                                span_angle=self._span_angle,
                                num_segments=self._num_segments)
        if not self._color.is_blank:
            self.mesh.set_data(vertices=self._vertices,
                               color=self._color.rgba)
        if not self._border_color.is_blank:
            self.border.set_data(pos=self._vertices[1:],
                                 color=self._border_color.rgba)
        
        self.update()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from ..shaders import FunctionChain
from .base_transform import BaseTransform
from .linear import NullTransform


class ChainTransform(BaseTransform):
    """
    BaseTransform subclass that performs a sequence of transformations in
    order. Internally, this class uses shaders.FunctionChain to generate
    its glsl_map and glsl_imap functions.

    Arguments:

    transforms : list of BaseTransform instances
        See ``transforms`` property.
    """
    glsl_map = None
    glsl_imap = None

    Linear = False
    Orthogonal = False
    NonScaling = False
    Isometric = False

    def __init__(self, *transforms):
        super(ChainTransform, self).__init__()

        # Set input transforms
        trs = []
        for tr in transforms:
            if isinstance(tr, (tuple, list)):
                trs.extend(tr)
            else:
                trs.append(tr)
        self._transforms = trs

        # ChainTransform does not have shader maps
        self._shader_map = None
        self._shader_imap = None

    @property
    def transforms(self):
        """ The list of transform that make up the transform chain.
        
        The order of transforms is given such that the last transform in the 
        list is the first to be invoked when mapping coordinates through 
        the chain. 
        
        For example, the following two mappings are equivalent::
        
            # Map coordinates through individual transforms:
            trans1 = STTransform(scale=(2, 3), translate=(0, 1))
            trans2 = PolarTransform()
            mapped = trans1.map(trans2.map(coords))
            
            # Equivalent mapping through chain:
            chain = ChainTransform([trans1, trans2])
            mapped = chain.map(coords)
            
        """
        return self._transforms

#     @transforms.setter
#     def transforms(self, tr):
#         #if self._enabled:
#             #raise RuntimeError("Shader is already enabled; cannot modify.")
#         if not isinstance(tr, list):
#             raise TypeError("Transform chain must be a list")
#         self._transforms = tr

    @property
    def Linear(self):
        b = True
        for tr in self._transforms:
            b &= tr.Linear
        return b

    @property
    def Orthogonal(self):
        b = True
        for tr in self._transforms:
            b &= tr.Orthogonal
        return b

    @property
    def NonScaling(self):
        b = True
        for tr in self._transforms:
            b &= tr.NonScaling
        return b

    @property
    def Isometric(self):
        b = True
        for tr in self._transforms:
            b &= tr.Isometric
        return b

    def map(self, coords):
        """Map coordinates

        Parameters
        ----------
        coords : array-like
            Coordinates to map.

        Returns
        -------
        coords : ndarray
            Coordinates.
        """
        for tr in reversed(self.transforms):
            coords = tr.map(coords)
        return coords

    def imap(self, coords):
        """Inverse map coordinates

        Parameters
        ----------
        coords : array-like
            Coordinates to inverse map.

        Returns
        -------
        coords : ndarray
            Coordinates.
        """
        for tr in self.transforms:
            coords = tr.imap(coords)
        return coords

    def shader_map(self):
        if self._shader_map is None:
            self._shader_map = self._make_shader_map(imap=False)
        else:
            for tr in self._transforms:
                tr.shader_map()  # force transform to update its shader
        return self._shader_map

    def shader_imap(self):
        if self._shader_imap is None:
            self._shader_imap = self._make_shader_map(imap=True)
        else:
            for tr in self._transforms:
                tr.shader_imap()  # force transform to update its shader
        return self._shader_imap

    def _make_shader_map(self, imap):
        if bool(imap):
            funcs = [tr.shader_imap() for tr in self.transforms]
        else:
            funcs = [tr.shader_map() for tr in reversed(self.transforms)]

        name = "transform_%s_chain" % ('imap' if bool(imap) else 'map')
        return FunctionChain(name, funcs)

    def flat(self):
        """
        Return a simplified chain by expanding any nested chains.
        """
        transforms = self.transforms[:]
        new_chain = []
        while len(transforms) > 0:
            tr = transforms.pop(0)
            if isinstance(tr, ChainTransform):
                transforms = tr.transforms[:] + transforms
            else:
                new_chain.append(tr)
        
        return ChainTransform(new_chain)

    def simplified(self):
        """
        Return a simplified chain by joining adjacent transforms.
        If the result is a single transform, return that transform.
        """
        tr = self.flat()
        if len(tr.transforms) == 0:
            return NullTransform()
        cont = True
        tr = tr.transforms
        while cont:
            new_tr = [tr[0]]
            cont = False
            for t2 in tr[1:]:
                t1 = new_tr[-1]
                pr = t1 * t2
                if not isinstance(pr, ChainTransform):
                    cont = True
                    new_tr.pop()
                    new_tr.append(pr)
                else:
                    new_tr.append(t2)
            tr = new_tr

        if len(tr) == 1:
            return tr[0]
        else:
            return ChainTransform(tr)

    def append(self, tr):
        """
        Add a new transform to the end of this chain.

        Parameters
        ----------
        tr : instance of Transform
            The transform to use.
        """
        self.transforms.append(tr)
        self.update()
        # Keep simple for now. Let's look at efficienty later
        # I feel that this class should not decide when to compose transforms
#         while len(self.transforms) > 0:
#             pr = tr * self.transforms[-1]
#             if isinstance(pr, ChainTransform):
#                 self.transforms.append(tr)
#                 break
#             else:
#                 self.transforms.pop()
#                 tr = pr
#                 if len(self.transforms)  == 0:
#                     self._transforms = [pr]
#                     break

    def prepend(self, tr):
        """
        Add a new transform to the beginning of this chain.

        Parameters
        ----------
        tr : instance of Transform
            The transform to use.
        """
        self.transforms.insert(0, tr)
        self.update()
        # Keep simple for now. Let's look at efficienty later
#         while len(self.transforms) > 0:
#             pr = self.transforms[0] * tr
#             if isinstance(pr, ChainTransform):
#                 self.transforms.insert(0, tr)
#                 break
#             else:
#                 self.transforms.pop(0)
#                 tr = pr
#                 if len(self.transforms)  == 0:
#                     self._transforms = [pr]
#                     break

    def __setitem__(self, index, tr):
        self._transforms[index] = tr
        if self._shader_map is not None:
            self._shader_map[-(index+1)] = tr.shader_map()
        if self._shader_imap is not None:
            self._shader_imap[index] = tr.shader_imap()
        self.update()

    def __mul__(self, tr):
        if isinstance(tr, ChainTransform):
            trs = tr.transforms
        else:
            trs = [tr]
        return ChainTransform(self.transforms+trs)

    def __rmul__(self, tr):
        if isinstance(tr, ChainTransform):
            trs = tr.transforms
        else:
            trs = [tr]
        return ChainTransform(trs+self.transforms)

    def __str__(self):
        names = [tr.__class__.__name__ for tr in self.transforms]
        return "<ChainTransform [%s] at 0x%x>" % (", ".join(names), id(self))
    
    def __repr__(self):
        tr = ",\n                 ".join(map(repr, self.transforms))
        return "<ChainTransform [%s] at 0x%x>" % (tr, id(self))



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from ..gloo import set_state, Texture2D
from ..color import get_colormap
from .shaders import ModularProgram, Function, FunctionChain
from .transforms import NullTransform
from .visual import Visual
from ..ext.six import string_types


VERT_SHADER = """
attribute vec2 a_position;
attribute vec2 a_texcoord;
varying vec2 v_texcoord;

void main() {
    v_texcoord = a_texcoord;
    gl_Position = $transform(vec4(a_position, 0., 1.));
}
"""

FRAG_SHADER = """
uniform sampler2D u_texture;
varying vec2 v_texcoord;

void main()
{
    vec2 texcoord = $map_uv_to_tex(vec4(v_texcoord, 0, 1)).xy;
    if(texcoord.x < 0.0 || texcoord.x > 1.0 ||
       texcoord.y < 0.0 || texcoord.y > 1.0) {
        discard;
    }
    gl_FragColor = $color_transform(texture2D(u_texture, texcoord));
}
"""  # noqa

_null_color_transform = 'vec4 pass(vec4 color) { return color; }'
_c2l = 'float cmap(vec4 color) { return (color.r + color.g + color.b) / 3.; }'


class ImageVisual(Visual):
    """Visual subclass displaying an image.

    Parameters
    ----------
    data : ndarray
        ImageVisual data. Can be shape (M, N), (M, N, 3), or (M, N, 4).
    method : str
        Selects method of rendering image in case of non-linear transforms.
        Each method produces similar results, but may trade efficiency
        and accuracy. If the transform is linear, this parameter is ignored
        and a single quad is drawn around the area of the image.

            * 'auto': Automatically select 'impostor' if the image is drawn
              with a nonlinear transform; otherwise select 'subdivide'.
            * 'subdivide': ImageVisual is represented as a grid of triangles
              with texture coordinates linearly mapped.
            * 'impostor': ImageVisual is represented as a quad covering the
              entire view, with texture coordinates determined by the
              transform. This produces the best transformation results, but may
              be slow.

    grid: tuple (rows, cols)
        If method='subdivide', this tuple determines the number of rows and
        columns in the image grid.
    cmap : str | ColorMap
        Colormap to use for luminance images.
    clim : str | tuple
        Limits to use for the colormap. Can be 'auto' to auto-set bounds to
        the min and max of the data.
    **kwargs : dict
        Keyword arguments to pass to `Visual`.

    Notes
    -----
    The colormap functionality through ``cmap`` and ``clim`` are only used
    if the data are 2D.
    """
    def __init__(self, data=None, method='auto', grid=(10, 10),
                 cmap='cubehelix', clim='auto', **kwargs):
        super(ImageVisual, self).__init__(**kwargs)
        self._program = ModularProgram(VERT_SHADER, FRAG_SHADER)
        self.clim = clim
        self.cmap = cmap

        self._data = None

        self._texture = None
        self._interpolation = 'nearest'
        if data is not None:
            self.set_data(data)

        self._method = method
        self._method_used = None
        self._grid = grid
        self._need_vertex_update = True

    def set_data(self, image):
        """Set the data

        Parameters
        ----------
        image : array-like
            The image data.
        """
        data = np.asarray(image)
        if self._data is None or self._data.shape != data.shape:
            self._need_vertex_update = True
        self._data = data
        self._texture = None

    @property
    def clim(self):
        return (self._clim if isinstance(self._clim, string_types) else
                tuple(self._clim))

    @clim.setter
    def clim(self, clim):
        if isinstance(clim, string_types):
            if clim != 'auto':
                raise ValueError('clim must be "auto" if a string')
        else:
            clim = np.array(clim, float)
            if clim.shape != (2,):
                raise ValueError('clim must have two elements')
        self._clim = clim
        self._need_vertex_update = True
        self.update()

    @property
    def cmap(self):
        return self._cmap

    @cmap.setter
    def cmap(self, cmap):
        self._cmap = get_colormap(cmap)
        self.update()

    @property
    def method(self):
        return self._method
    
    @method.setter
    def method(self, m):
        if self._method != m:
            self._method = m
            self._need_vertex_update = True
            self.update()

    @property
    def size(self):
        return self._data.shape[:2][::-1]

    def _build_vertex_data(self, transforms):
        method = self._method
        grid = self._grid
        if method == 'auto':
            if transforms.get_full_transform().Linear:
                method = 'subdivide'
                grid = (1, 1)
            else:
                method = 'impostor'
        self._method_used = method

        # TODO: subdivision and impostor modes should be handled by new
        # components?
        if method == 'subdivide':
            # quads cover area of image as closely as possible
            w = 1.0 / grid[1]
            h = 1.0 / grid[0]

            quad = np.array([[0, 0, 0], [w, 0, 0], [w, h, 0],
                             [0, 0, 0], [w, h, 0], [0, h, 0]],
                            dtype=np.float32)
            quads = np.empty((grid[1], grid[0], 6, 3), dtype=np.float32)
            quads[:] = quad

            mgrid = np.mgrid[0.:grid[1], 0.:grid[0]].transpose(1, 2, 0)
            mgrid = mgrid[:, :, np.newaxis, :]
            mgrid[..., 0] *= w
            mgrid[..., 1] *= h

            quads[..., :2] += mgrid
            tex_coords = quads.reshape(grid[1]*grid[0]*6, 3)
            tex_coords = np.ascontiguousarray(tex_coords[:, :2])
            vertices = tex_coords * self.size
            
            # vertex shader provides correct texture coordinates
            self._program.frag['map_uv_to_tex'] = NullTransform()
        
        elif method == 'impostor':
            # quad covers entire view; frag. shader will deal with image shape
            vertices = np.array([[-1, -1], [1, -1], [1, 1],
                                 [-1, -1], [1, 1], [-1, 1]],
                                dtype=np.float32)
            tex_coords = vertices

            # vertex shader provides ND coordinates; 
            # fragment shader maps to texture coordinates
            self._program.vert['transform'] = NullTransform()
            self._raycast_func = Function('''
                vec4 map_local_to_tex(vec4 x) {
                    // Cast ray from 3D viewport to surface of image
                    // (if $transform does not affect z values, then this
                    // can be optimized as simply $transform.map(x) )
                    vec4 p1 = $transform(x);
                    vec4 p2 = $transform(x + vec4(0, 0, 0.5, 0));
                    p1 /= p1.w;
                    p2 /= p2.w;
                    vec4 d = p2 - p1;
                    float f = p2.z / d.z;
                    vec4 p3 = p2 - d * f;
                    
                    // finally map local to texture coords
                    return vec4(p3.xy / $image_size, 0, 1);
                }
            ''')
            self._raycast_func['image_size'] = self.size
            self._program.frag['map_uv_to_tex'] = self._raycast_func
        
        else:
            raise ValueError("Unknown image draw method '%s'" % method)
        
        self._program['a_position'] = vertices.astype(np.float32)
        self._program['a_texcoord'] = tex_coords.astype(np.float32)
        self._need_vertex_update = False

    def _build_texture(self):
        data = self._data
        if data.dtype == np.float64:
            data = data.astype(np.float32)

        if data.ndim == 2 or data.shape[2] == 1:
            # deal with clim on CPU b/c of texture depth limits :(
            # can eventually do this by simulating 32-bit float... maybe
            clim = self._clim
            if isinstance(clim, string_types) and clim == 'auto':
                clim = np.min(data), np.max(data)
            clim = np.asarray(clim, dtype=np.float32)
            data = data - clim[0]  # not inplace so we don't modify orig data
            if clim[1] - clim[0] > 0:
                data /= clim[1] - clim[0]
            else:
                data[:] = 1 if data[0, 0] != 0 else 0
            fun = FunctionChain(None, [Function(_c2l),
                                       Function(self.cmap.glsl_map)])
            self._clim = np.array(clim)
        else:
            fun = Function(_null_color_transform)
        self._program.frag['color_transform'] = fun
        self._texture = Texture2D(data, interpolation=self._interpolation)
        self._program['u_texture'] = self._texture

    def bounds(self, mode, axis):
        """Get the bounds

        Parameters
        ----------
        mode : str
            Describes the type of boundary requested. Can be "visual", "data",
            or "mouse".
        axis : 0, 1, 2
            The axis along which to measure the bounding values, in
            x-y-z order.
        """
        if axis > 1:
            return (0, 0)
        else:
            return (0, self.size[axis])

    def draw(self, transforms):
        """Draw the visual

        Parameters
        ----------
        transforms : instance of TransformSystem
            The transforms to use.
        """
        if self._data is None:
            return

        set_state(cull_face=False)

        # upload texture is needed
        if self._texture is None:
            self._build_texture()

        # rebuild vertex buffers if needed
        if self._need_vertex_update:
            self._build_vertex_data(transforms)

        # update transform
        method = self._method_used
        if method == 'subdivide':
            self._program.vert['transform'] = transforms.get_full_transform()
        else:
            self._raycast_func['transform'] = \
                transforms.get_full_transform().inverse

        self._program.draw('triangles')



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
"""
The classes in scene.visuals are visuals that may be added to a scenegraph
using the methods and properties defined in `vispy.scene.Node` such as name,
visible, parent, children, etc...

These classes are automatically generated by mixing `vispy.scene.Node` with
the Visual classes found in `vispy.visuals`.

For developing custom visuals, it is recommended to subclass from
`vispy.visuals.Visual` rather than `vispy.scene.Node`.
"""
import re

from .. import visuals
from .node import Node


def create_visual_node(subclass):
    # Create a new subclass of Node.
    
    # Decide on new class name
    clsname = subclass.__name__
    assert clsname.endswith('Visual')
    clsname = clsname[:-6]
    
    # Generate new docstring based on visual docstring
    try:
        doc = generate_docstring(subclass, clsname)
    except Exception:
        # If parsing fails, just return the original Visual docstring
        doc = subclass.__doc__
    
    # New __init__ method
    def __init__(self, *args, **kwargs):
        parent = kwargs.pop('parent', None)
        name = kwargs.pop('name', None)
        self.name = name  # to allow __str__ before Node.__init__
        subclass.__init__(self, *args, **kwargs)
        Node.__init__(self, parent=parent, name=name)
    
    # Create new class
    cls = type(clsname, (subclass, Node), {'__init__': __init__, 
                                           '__doc__': doc})
    return cls


def generate_docstring(subclass, clsname):
    # Generate a Visual+Node docstring by modifying the Visual's docstring
    # to include information about Node inheritance and extra init args.
    
    sc_doc = subclass.__doc__
    if sc_doc is None:
        sc_doc = ""
        
    # find locations within docstring to insert new parameters
    lines = sc_doc.split("\n")
    
    # discard blank lines at start
    while lines and lines[0].strip() == '':
        lines.pop(0)

    i = 0
    params_started = False
    param_indent = None
    first_blank = None
    param_end = None
    while i < len(lines):
        line = lines[i]
        # ignore blank lines and '------' lines
        if re.search(r'\w', line):  
            indent = len(line) - len(line.lstrip())
            # If Params section has already started, check for end of params
            # (that is where we will insert new params)
            if params_started:
                if indent < param_indent:
                    break
                elif indent == param_indent:
                    # might be end of parameters block..
                    if re.match(r'\s*[a-zA-Z0-9_]+\s*:\s*\S+', line) is None:
                        break
                param_end = i + 1
            
            # Check for beginning of params section
            elif re.match(r'\s*Parameters\s*', line):
                params_started = True
                param_indent = indent
                if first_blank is None:
                    first_blank = i
        
        # Check for first blank line
        # (this is where the Node inheritance description will be 
        # inserted)
        elif first_blank is None and line.strip() == '':
            first_blank = i

        i += 1
        if i == len(lines) and param_end is None:
            # reached end of docstring; insert here
            param_end = i

    # If original docstring has no params heading, we need to generate it.
    if not params_started:
        lines.extend(["", "    Parameters", "    ----------"])
        param_end = len(lines)
        if first_blank is None:
            first_blank = param_end - 3
        params_started = True
    
    # build class and parameter description strings
    class_desc = ("\n    This class inherits from visuals.%sVisual and "
                  "scene.Node, allowing the visual to be placed inside a "
                  "scenegraph.\n" % (clsname))
    parm_doc = ("    parent : Node\n"
                "        The parent node to assign to this node (optional).\n"
                "    name : string\n"
                "        A name for this node, used primarily for debugging\n"
                "        (optional).")
    
    # assemble all docstring parts
    lines = (lines[:first_blank] +
             [class_desc] +
             lines[first_blank:param_end] +
             [parm_doc] +
             lines[param_end:])
            
    doc = '\n'.join(lines)
    return doc


__all__ = []

for obj_name in dir(visuals):
    obj = getattr(visuals, obj_name)
    if (isinstance(obj, type) and 
       issubclass(obj, visuals.Visual) and 
       obj is not visuals.Visual):
        cls = create_visual_node(obj)
        globals()[cls.__name__] = cls
        __all__.append(cls.__name__)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

"""
Normal components are modular shader components used for retrieving or
generating surface normal vectors.

These components generate a function in the fragment shader that accepts no
arguments and returns a vec4 normal vector. Typically, the normal vector
is computed in the vertex shader and passed by varying to the fragment
shader.
"""

from __future__ import division

from .component import VisualComponent
from ..shaders import Varying
from ... import gloo


class VertexNormalComponent(VisualComponent):
    SHADERS = dict(
        frag_normal="""
            vec4 normal() {
                return $norm;
            }
        """,
        vert_post_hook="""
            void normal_support() {
                //vec3 o = vec3(0,0,0);
                //vec3 i = o + $input_normal.xyz;
                //$output_normal = $map_local_to_nd(vec4(i,1)) -
                //                 $map_local_to_nd(vec4(o,1));
                $output_normal = vec4($input_normal, 1);
            }
        """)

    # exclude frag_normal when auto-attaching shaders because the visual
    # does not have a 'frag_normal' hook; instead this function will be called
    # by another component.
    AUTO_ATTACH = ['vert_post_hook']

    def __init__(self, meshdata, smooth=True):
        super(VertexNormalComponent, self).__init__()
        self._meshdata = meshdata
        self.smooth = smooth
        self._vbo = None
        self._vbo_mode = None

        # Create Varying to connect vertex / fragment shaders
        var = Varying('norm', dtype='vec4')
        self._funcs['frag_normal']['norm'] = var
        self._funcs['vert_post_hook']['output_normal'] = var

    def _make_vbo(self, mode):
        if self._vbo is None or self._vbo_mode != mode:
            if mode is self.DRAW_PRE_INDEXED:
                index = 'faces'
            else:
                index = None
            if self.smooth:
                norm = self._meshdata.get_vertex_normals(indexed=index)
            else:
                if index != 'faces':
                    raise Exception("Not possible to draw faceted mesh without"
                                    "pre-indexing.")
                norm = self._meshdata.get_face_normals(indexed=index)
            self._vbo = gloo.VertexBuffer(norm)
            self._vbo_mode = mode
        return self._vbo

    def normal_shader(self):
        """
        Return the fragment shader function that returns a normal vector.
        """
        return self._funcs['frag_normal']

    def activate(self, program, mode):
        vf = self._funcs['vert_post_hook']
        vf['input_normal'] = self._make_vbo(mode)  # attribute vec4
        vf['map_local_to_nd'] = self.visual._program.vert['map_local_to_nd']

    @property
    def supported_draw_modes(self):
        if self.smooth:
            return set([self.DRAW_PRE_INDEXED, self.DRAW_UNINDEXED])
        else:
            # not possible to draw faceted mesh without pre-indexing.
            return set([self.DRAW_PRE_INDEXED])



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import numpy as np

from .widget import Widget


class Grid(Widget):
    """
    Widget that automatically sets the position and size of child Widgets to
    proportionally divide its internal area into a grid.

    Parameters
    ----------
    spacing : int
        Spacing between widgets.
    **kwargs : dict
        Keyword arguments to pass to `Widget`.
    """
    def __init__(self, spacing=6, **kwargs):
        from .viewbox import ViewBox
        self._next_cell = [0, 0]  # row, col
        self._cells = {}
        self._grid_widgets = {}
        self.spacing = spacing
        self._n_added = 0
        self._default_class = ViewBox  # what to add when __getitem__ is used
        Widget.__init__(self, **kwargs)

    def __getitem__(self, idxs):
        """Return an item or create it if the location is available"""
        if not isinstance(idxs, tuple):
            idxs = (idxs,)
        if len(idxs) == 1:
            idxs = idxs + (slice(None),)
        elif len(idxs) != 2:
            raise ValueError('Incorrect index: %s' % (idxs,))
        lims = np.empty((2, 2), int)
        for ii, idx in enumerate(idxs):
            if isinstance(idx, int):
                idx = slice(idx, idx + 1, None)
            if not isinstance(idx, slice):
                raise ValueError('indices must be slices or integers, not %s'
                                 % (type(idx),))
            if idx.step is not None and idx.step != 1:
                raise ValueError('step must be one or None, not %s' % idx.step)
            start = 0 if idx.start is None else idx.start
            end = self.grid_size[ii] if idx.stop is None else idx.stop
            lims[ii] = [start, end]
        layout = self.layout_array
        existing = layout[lims[0, 0]:lims[0, 1], lims[1, 0]:lims[1, 1]] + 1
        if existing.any():
            existing = set(list(existing.ravel()))
            ii = list(existing)[0] - 1
            if len(existing) != 1 or ((layout == ii).sum() !=
                                      np.prod(np.diff(lims))):
                raise ValueError('Cannot add widget (collision)')
            return self._grid_widgets[ii][-1]
        spans = np.diff(lims)[:, 0]
        item = self.add_widget(self._default_class(),
                               row=lims[0, 0], col=lims[1, 0],
                               row_span=spans[0], col_span=spans[1])
        return item

    def add_widget(self, widget=None, row=None, col=None, row_span=1,
                   col_span=1):
        """
        Add a new widget to this grid. This will cause other widgets in the
        grid to be resized to make room for the new widget.

        Parameters
        ----------
        widget : Widget
            The Widget to add
        row : int
            The row in which to add the widget (0 is the topmost row)
        col : int
            The column in which to add the widget (0 is the leftmost column)
        row_span : int
            The number of rows to be occupied by this widget. Default is 1.
        col_span : int
            The number of columns to be occupied by this widget. Default is 1.

        Notes
        -----
        The widget's parent is automatically set to this grid, and all other
        parent(s) are removed.
        """
        if row is None:
            row = self._next_cell[0]
        if col is None:
            col = self._next_cell[1]

        if widget is None:
            widget = Widget()

        _row = self._cells.setdefault(row, {})
        _row[col] = widget
        self._grid_widgets[self._n_added] = (row, col, row_span, col_span,
                                             widget)
        self._n_added += 1
        widget.parent = self

        self._next_cell = [row, col+col_span]
        self._update_child_widgets()
        return widget

    def add_grid(self, row=None, col=None, row_span=1, col_span=1,
                 **kwargs):
        """
        Create a new Grid and add it as a child widget.

        Parameters
        ----------
        row : int
            The row in which to add the widget (0 is the topmost row)
        col : int
            The column in which to add the widget (0 is the leftmost column)
        row_span : int
            The number of rows to be occupied by this widget. Default is 1.
        col_span : int
            The number of columns to be occupied by this widget. Default is 1.
        **kwargs : dict
            Keyword arguments to pass to the new `Grid`.
        """
        from .grid import Grid
        grid = Grid(**kwargs)
        return self.add_widget(grid, row, col, row_span, col_span)

    def add_view(self, row=None, col=None, row_span=1, col_span=1,
                 **kwargs):
        """
        Create a new ViewBox and add it as a child widget.

        Parameters
        ----------
        row : int
            The row in which to add the widget (0 is the topmost row)
        col : int
            The column in which to add the widget (0 is the leftmost column)
        row_span : int
            The number of rows to be occupied by this widget. Default is 1.
        col_span : int
            The number of columns to be occupied by this widget. Default is 1.
        **kwargs : dict
            Keyword arguments to pass to `ViewBox`.
        """
        from .viewbox import ViewBox
        view = ViewBox(**kwargs)
        return self.add_widget(view, row, col, row_span, col_span)

    def next_row(self):
        self._next_cell = [self._next_cell[0] + 1, 0]

    @property
    def grid_size(self):
        rvals = [widget[0]+widget[2] for widget in self._grid_widgets.values()]
        cvals = [widget[1]+widget[3] for widget in self._grid_widgets.values()]
        return max(rvals + [0]), max(cvals + [0])

    @property
    def layout_array(self):
        locs = -1 * np.ones(self.grid_size, int)
        for key in self._grid_widgets.keys():
            r, c, rs, cs = self._grid_widgets[key][:4]
            locs[r:r + rs, c:c + cs] = key
        return locs

    def __repr__(self):
        return (('<Grid at %s:\n' % hex(id(self))) +
                str(self.layout_array + 1) + '>')

    def _update_child_widgets(self):
        # Resize all widgets in this grid to share space.
        # This logic will need a lot of work..
        n_rows, n_cols = self.grid_size
        if n_rows == 0:
            return
        # determine starting/ending position of each row and column
        s2 = self.spacing / 2.
        rect = self.rect.padded(self.padding + self.margin - s2)
        rows = np.linspace(rect.bottom, rect.top, n_rows+1)
        rowstart = rows[:-1] + s2
        rowend = rows[1:] - s2
        cols = np.linspace(rect.left, rect.right, n_cols+1)
        colstart = cols[:-1] + s2
        colend = cols[1:] - s2

        # snap to pixel boundaries to avoid drawing artifacts
        colstart = np.round(colstart)
        colend = np.round(colend)
        rowstart = np.round(rowstart)
        rowend = np.round(rowend)

        for key in self._grid_widgets.keys():
            row, col, rspan, cspan, ch = self._grid_widgets[key]

            # Translate the origin of the node to the corner of the area
            # ch.transform.reset()
            # ch.transform.translate((colstart[col], rowstart[row]))
            ch.pos = colstart[col], rowstart[row]

            # ..and set the size to match.
            w = colend[col+cspan-1]-colstart[col]
            h = rowend[row+rspan-1]-rowstart[row]
            ch.size = w, h



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.


"""
RegularPolygonVisual visual based on EllipseVisual
"""

from __future__ import division

from ..color import Color
from .ellipse import EllipseVisual


class RegularPolygonVisual(EllipseVisual):
    """
    Displays a regular polygon

    Parameters
    ----------

    pos : array
        Center of the regular polygon
    color : str | tuple | list of colors
        Fill color of the polygon
    border_color : str | tuple | list of colors
        Border color of the polygon
    radius : float
        Radius of the regular polygon
        Defaults to  0.1
    sides : int
        Number of sides of the regular polygon
    """
    def __init__(self, pos=None, color='black', border_color=None,
                 radius=0.1, sides=4, **kwargs):
        super(EllipseVisual, self).__init__()
        self.mesh.mode = 'triangle_fan'
        self._pos = pos
        self._color = Color(color)
        self._border_color = Color(border_color)
        self._radius = radius
        self._sides = sides
        self._update()

    @property
    def sides(self):
        """ The number of sides in the regular polygon.
        """
        return self._sides

    @sides.setter
    def sides(self, sides):
        if sides < 3:
            raise ValueError('PolygonVisual must have at least 3 sides, not %s'
                             % sides)
        self._sides = sides
        self._update()

    def _update(self):
        if self._pos is None:
            return
        self._generate_vertices(pos=self._pos, radius=self._radius,
                                start_angle=0.,
                                span_angle=360.,
                                num_segments=self._sides)
        
        if not self._color.is_blank:
            self.mesh.set_data(vertices=self._vertices, 
                               color=self._color.rgba)
        if not self._border_color.is_blank:
            self.border.set_data(pos=self._vertices[1:],
                                 color=self._border_color.rgba)

        self.update()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from ..util.event import Event, EmitterGroup
from ..visuals.transforms import (NullTransform, BaseTransform, 
                                  ChainTransform, create_transform)


class Node(object):
    """ Base class representing an object in a scene.

    A group of nodes connected through parent-child relationships define a 
    scenegraph. Nodes may have any number of children or parents, although 
    it is uncommon to have more than one parent.

    Each Node defines a ``transform`` property, which describes the position,
    orientation, scale, etc. of the Node relative to its parent. The Node's
    children inherit this property, and then further apply their own
    transformations on top of that. 
    
    With the ``transform`` property, each Node implicitly defines a "local" 
    coordinate system, and the Nodes and edges in the scenegraph can be though
    of as coordinate systems connected by transformation functions.
    
    Parameters
    ----------
    parent : Node
        The parent of the Node.
    name : str
        The name used to identify the node.
    """
    
    # Needed to allow subclasses to repr() themselves before Node.__init__()
    _name = None

    def __init__(self, parent=None, name=None):
        self.name = name
        self._visible = True

        # Add some events to the emitter groups:
        events = ['parents_change', 'children_change', 'transform_change',
                  'mouse_press', 'mouse_move', 'mouse_release', 'mouse_wheel', 
                  'key_press', 'key_release']
        # Create event emitter if needed (in subclasses that inherit from
        # Visual, we already have an emitter to share)
        if not hasattr(self, 'events'):
            self.events = EmitterGroup(source=self, auto_connect=True,
                                       update=Event)
        self.events.add(**dict([(ev, Event) for ev in events]))
        
        # Entities are organized in a parent-children hierarchy
        self._children = []
        # TODO: use weakrefs for parents.
        self._parents = []
        if parent is not None:
            self.parents = parent
            
        self._document = None

        # Components that all entities in vispy have
        # todo: default transform should be trans-scale-rot transform
        self._transform = NullTransform()
    
    # todo: move visible to BaseVisualNode class when we make Node not a Visual
    @property
    def visible(self):
        """ Whether this node should be drawn or not. Only applicable to
        nodes that can be drawn.
        """
        return self._visible
    
    @visible.setter
    def visible(self, val):
        self._visible = bool(val)
        self.update()
    
    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        self._name = n

    @property
    def children(self):
        """ A copy of the list of children of this node. Do not add
        items to this list, but use ``x.parent = y`` instead.
        """
        return list(self._children)

    @property
    def parent(self):
        """ Get/set the parent. If the node has multiple parents while
        using this property as a getter, an error is raised.
        """
        if not self._parents:
            return None
        elif len(self._parents) == 1:
            return self._parents[0]
        else:
            raise RuntimeError('Ambiguous parent: there are multiple parents.')

    @parent.setter
    def parent(self, parent):
        # This is basically an alias
        self.parents = parent

    @property
    def parents(self):
        """ Get/set a tuple of parents.
        """
        return tuple(self._parents)

    @parents.setter
    def parents(self, parents):
        # Test input
        if isinstance(parents, Node):
            parents = (parents,)
        if not hasattr(parents, '__iter__'):
            raise ValueError("Node.parents must be iterable (got %s)"
                             % type(parents))

        # Test that all parents are entities
        for p in parents:
            if not isinstance(p, Node):
                raise ValueError('A parent of an node must be an node too,'
                                 ' not %s.' % p.__class__.__name__)

        # Apply
        prev = list(self._parents)  # No list.copy() on Py2.x
        with self.events.parents_change.blocker():
            # Remove parents
            for parent in prev:
                if parent not in parents:
                    self.remove_parent(parent)
            # Add new parents
            for parent in parents:
                if parent not in prev:
                    self.add_parent(parent)

        self.events.parents_change(new=parents, old=prev)

    def add_parent(self, parent):
        """Add a parent

        Parameters
        ----------
        parent : instance of Node
            The parent.
        """
        if parent in self._parents:
            return
        self._parents.append(parent)
        parent._add_child(self)
        self.events.parents_change(added=parent)
        self.update()

    def remove_parent(self, parent):
        """Remove a parent

        Parameters
        ----------
        parent : instance of Node
            The parent.
        """
        if parent not in self._parents:
            raise ValueError("Parent not in set of parents for this node.")
        self._parents.remove(parent)
        parent._remove_child(self)
        self.events.parents_change(removed=parent)

    def _add_child(self, node):
        self._children.append(node)
        self.events.children_change(added=node)
        node.events.update.connect(self.events.update)

    def _remove_child(self, node):
        self._children.remove(node)
        self.events.children_change(removed=node)
        node.events.update.disconnect(self.events.update)

    def update(self):
        """
        Emit an event to inform listeners that properties of this Node or its
        children have changed.
        """
        self.events.update()

    @property
    def document(self):
        """ The document is an optional property that is an node representing
        the coordinate system from which this node should make physical 
        measurements such as px, mm, pt, in, etc. This coordinate system 
        should be used when determining line widths, font sizes, and any
        other lengths specified in physical units.
        
        The default is None; in this case, a default document is used during
        drawing (usually this is supplied by the SceneCanvas).
        """
        return self._document
    
    @document.setter
    def document(self, doc):
        if doc is not None and not isinstance(doc, Node):
            raise TypeError("Document property must be Node or None.")
        self._document = doc
        self.update()

    @property
    def transform(self):
        """ The transform that maps the local coordinate frame to the
        coordinate frame of the parent.
        """
        return self._transform

    @transform.setter
    def transform(self, tr):
        if self._transform is not None:
            self._transform.changed.disconnect(self._transform_changed)
        assert isinstance(tr, BaseTransform)
        self._transform = tr
        self._transform.changed.connect(self._transform_changed)
        self._transform_changed(None)

    def set_transform(self, type_, *args, **kwargs):
        """ Create a new transform of *type* and assign it to this node.

        All extra arguments are used in the construction of the transform.

        Parameters
        ----------
        type_ : str
            The transform type.
        *args : tuple
            Arguments.
        **kwargs : dict
            Keywoard arguments.
        """
        self.transform = create_transform(type_, *args, **kwargs)

    def _transform_changed(self, event):
        self.events.transform_change()
        self.update()

    def _parent_chain(self):
        """
        Return the chain of parents starting from this node. The chain ends
        at the first node with either no parents or multiple parents.
        """
        chain = [self]
        while True:
            try:
                parent = chain[-1].parent
            except Exception:
                break
            if parent is None:
                break
            chain.append(parent)
        return chain

    def describe_tree(self, with_transform=False):
        """Create tree diagram of children

        Parameters
        ----------
        with_transform : bool
            If true, add information about node transform types.

        Returns
        ----------
        tree : str
            The tree diagram.
        """
        # inspired by https://github.com/mbr/asciitree/blob/master/asciitree.py
        return self._describe_tree('', with_transform)

    def _describe_tree(self, prefix, with_transform):
        """Helper function to actuall construct the tree"""
        extra = ': "%s"' % self.name if self.name is not None else ''
        if with_transform:
            extra += (' [%s]' % self.transform.__class__.__name__)
        output = ''
        if len(prefix) > 0:
            output += prefix[:-3]
            output += '  +--'
        output += '%s%s\n' % (self.__class__.__name__, extra)

        n_children = len(self.children)
        for ii, child in enumerate(self.children):
            sub_prefix = prefix + ('   ' if ii+1 == n_children else '  |')
            output += child._describe_tree(sub_prefix, with_transform)
        return output

    def common_parent(self, node):
        """
        Return the common parent of two entities

        If the entities have no common parent, return None.
        Does not search past multi-parent branches.

        Parameters
        ----------
        node : instance of Node
            The other node.

        Returns
        -------
        parent : instance of Node | None
            The parent.
        """
        p1 = self._parent_chain()
        p2 = node._parent_chain()
        for p in p1:
            if p in p2:
                return p
        return None

    def node_path_to_child(self, node):
        """Return a list describing the path from this node to a child node

        This method assumes that the given node is a child node. Multiple
        parenting is allowed.

        Parameters
        ----------
        node : instance of Node
            The child node.

        Returns
        -------
        path : list | None
            The path.
        """
        if node is self:
            return []

        # Go up from the child node as far as we can
        path1 = [node]
        child = node
        while len(child.parents) == 1:
            child = child.parent
            path1.append(child)
            # Early exit
            if child is self:
                return list(reversed(path1))
        
        # Verify that we're not cut off
        if len(path1[-1].parents) == 0:
            raise RuntimeError('%r is not a child of %r' % (node, self))
        
        def _is_child(path, parent, child):
            path.append(parent)
            if child in parent.children:
                return path
            else:
                for c in parent.children:
                    possible_path = _is_child(path[:], c, child)
                    if possible_path:
                        return possible_path
            return None

        # Search from the parent towards the child
        path2 = _is_child([], self, path1[-1])
        if not path2:
            raise RuntimeError('%r is not a child of %r' % (node, self))

        # Return
        return path2 + list(reversed(path1))

    def node_path(self, node):
        """Return two lists describing the path from this node to another

        Parameters
        ----------
        node : instance of Node
            The other node.

        Returns
        -------
        p1 : list
            First path (see below).
        p2 : list
            Second path (see below).

        Notes
        -----
        The first list starts with this node and ends with the common parent
        between the endpoint nodes. The second list contains the remainder of
        the path from the common parent to the specified ending node.
        
        For example, consider the following scenegraph::
        
            A --- B --- C --- D
                   \
                    --- E --- F
        
        Calling `D.node_path(F)` will return::
        
            ([D, C, B], [E, F])
        
        Note that there must be a _single_ path in the scenegraph that connects
        the two entities; otherwise an exception will be raised.        
        """
        p1 = self._parent_chain()
        p2 = node._parent_chain()
        cp = None
        for p in p1:
            if p in p2:
                cp = p
                break
        if cp is None:
            raise RuntimeError("No single-path common parent between nodes %s "
                               "and %s." % (self, node))
        
        p1 = p1[:p1.index(cp)+1]
        p2 = p2[:p2.index(cp)][::-1]
        return p1, p2
        
    def node_path_transforms(self, node):
        """Return the list of transforms along the path to another node.
        
        The transforms are listed in reverse order, such that the last 
        transform should be applied first when mapping from this node to 
        the other.

        Parameters
        ----------
        node : instance of Node
            The other node.

        Returns
        -------
        transform : instance of Transform
            The transform.
        """
        a, b = self.node_path(node)
        return ([n.transform.inverse for n in b] +
                [n.transform for n in a[:-1]])[::-1]

    def node_transform(self, node):
        """
        Return the transform that maps from the coordinate system of
        *self* to the local coordinate system of *node*.

        Note that there must be a _single_ path in the scenegraph that connects
        the two entities; otherwise an exception will be raised.

        Parameters
        ----------
        node : instance of Node
            The other node.

        Returns
        -------
        transform : instance of ChainTransform
            The transform.
        """
        return ChainTransform(self.node_path_transforms(node))

    def __repr__(self):
        name = "" if self.name is None else " name="+self.name
        return "<%s%s at 0x%x>" % (self.__class__.__name__, name, id(self))



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
import numpy as np
from numpy.testing import assert_array_equal, assert_allclose

from vispy.testing import run_tests_if_main
from vispy.geometry import create_cube, create_cylinder, create_sphere


def test_cube():
    """Test cube function"""
    vertices, filled, outline = create_cube()
    assert_array_equal(np.arange(len(vertices)), np.unique(filled))
    assert_array_equal(np.arange(len(vertices)), np.unique(outline))


def test_sphere():
    """Test sphere function"""
    md = create_sphere(10, 20, radius=10)
    radii = np.sqrt((md.get_vertices() ** 2).sum(axis=1))
    assert_allclose(radii, np.ones_like(radii) * 10)


def test_cylinder():
    """Test cylinder function"""
    md = create_cylinder(10, 20, radius=[10, 10])
    radii = np.sqrt((md.get_vertices()[:, :2] ** 2).sum(axis=1))
    assert_allclose(radii, np.ones_like(radii) * 10)


run_tests_if_main()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .line import LineVisual
from .markers import MarkersVisual
from .visual import Visual


class LinePlotVisual(Visual):
    """Visual displaying a plot line with optional markers.

    Parameters
    ----------
    data : array-like
        Arguments can be passed as ``(Y,)``, ``(X, Y)`` or
        ``np.array((X, Y))``.
    color : instance of Color
        Color of the line.
    symbol : str
        Marker symbol to use.
    line_kind : str
        Kind of line to draw. For now, only solid lines (``'-'``)
        are supported.
    width : float
        Line width.
    marker_size : float
        Marker size. If `size == 0` markers will not be shown.
    edge_color : instance of Color
        Color of the marker edge.
    face_color : instance of Color
        Color of the marker face.
    edge_width : float
        Edge width of the marker.
    connect : str | array
        See LineVisual.
    **kwargs : keyword arguments
        Argements to pass to the super class.

    Examples
    --------
    All of these syntaxes will work:

        >>> LinePlotVisual(y_vals)
        >>> LinePlotVisual(x_vals, y_vals)
        >>> LinePlotVisual(xy_vals)

    See also
    --------
    LineVisual, MarkersVisual, marker_types
    """
    _line_kwargs = ('color', 'width', 'connect')
    _marker_kwargs = ('edge_color', 'face_color', 'edge_width',
                      'marker_size', 'symbol')
    _kw_trans = dict(marker_size='size')

    def __init__(self, data, color='k', symbol='o', line_kind='-',
                 width=1., marker_size=10., edge_color='k', face_color='w',
                 edge_width=1., connect='strip', **kwargs):
        Visual.__init__(self, **kwargs)
        if line_kind != '-':
            raise ValueError('Only solid lines currently supported')
        self._line = LineVisual()
        self._markers = MarkersVisual()
        self.set_data(data, color=color, symbol=symbol,
                      width=width, marker_size=marker_size,
                      edge_color=edge_color, face_color=face_color,
                      edge_width=edge_width, connect=connect)

    def set_data(self, data, **kwargs):
        """Set the line data

        Parameters
        ----------
        data : array-like
            The data.
        **kwargs : dict
            Keywoard arguments to pass to MarkerVisual and LineVisal.
        """
        pos = np.atleast_1d(data).astype(np.float32)
        if pos.ndim == 1:
            pos = pos[:, np.newaxis]
        elif pos.ndim > 2:
            raise ValueError('data must have at most two dimensions')

        if pos.size == 0:
            pos = self._line.pos

            # if both args and keywords are zero, then there is no
            # point in calling this function.
            if len(kwargs) == 0:
                raise TypeError("neither line points nor line properties"
                                "are provided")
        elif pos.shape[1] == 1:
            x = np.arange(pos.shape[0], dtype=np.float32)[:, np.newaxis]
            pos = np.concatenate((x, pos), axis=1)
        # if args are empty, don't modify position
        elif pos.shape[1] > 2:
            raise TypeError("Too many coordinates given (%s; max is 2)."
                            % pos.shape[1])

        # todo: have both sub-visuals share the same buffers.
        line_kwargs = {}
        for k in self._line_kwargs:
            if k in kwargs:
                k_ = self._kw_trans[k] if k in self._kw_trans else k
                line_kwargs[k] = kwargs.pop(k_)
        self._line.set_data(pos=pos, **line_kwargs)
        marker_kwargs = {}
        for k in self._marker_kwargs:
            if k in kwargs:
                k_ = self._kw_trans[k] if k in self._kw_trans else k
                marker_kwargs[k_] = kwargs.pop(k)
        self._markers.set_data(pos=pos, **marker_kwargs)
        if len(kwargs) > 0:
            raise TypeError("Invalid keyword arguments: %s" % kwargs.keys())

    def bounds(self, mode, axis):
        """Get the bounds

        Parameters
        ----------
        mode : str
            Describes the type of boundary requested. Can be "visual", "data",
            or "mouse".
        axis : 0, 1, 2
            The axis along which to measure the bounding values, in
            x-y-z order.
        """
        return self._line.bounds(mode, axis)

    def draw(self, transforms):
        """Draw the visual

        Parameters
        ----------
        transforms : instance of TransformSystem
            The transforms to use.
        """
        for v in self._line, self._markers:
            v.draw(transforms)



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from .color import UniformColorComponent, VertexColorComponent  # noqa
from .component import VisualComponent  # noqa
from .material import GridContourComponent, ShadingComponent  # noqa
from .normal import VertexNormalComponent  # noqa
from .texture import (TextureComponent, VertexTextureCoordinateComponent,  # noqa
                      TextureCoordinateComponent)  # noqa
from .vertex import XYPosComponent, XYZPosComponent, HeightFieldComponent  # noqa


from .clipper import Clipper  # noqa
from .color2 import Alpha, ColorFilter  # noqa



# -*- coding: utf-8 -*-
# Copyright (c) 2015, Vispy Development Team.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

from ..util.event import Event
from ..visuals.transforms import TransformCache, TransformSystem


class SceneEvent(Event, TransformSystem):
    """
    SceneEvent is an Event that tracks its path through a scenegraph,
    beginning at a Canvas. It exposes information useful during drawing
    and user interaction.
    """

    def __init__(self, type, canvas, transform_cache=None):
        Event.__init__(self, type=type)

        # Note that we are completely replacing the TransformSystem.__init__
        # implementation.
        self._canvas = canvas
        self._dpi = canvas.dpi

        # Init stacks
        self._stack = []  # list of entities
        self._stack_ids = set()
        self._viewbox_stack = []
        self._doc_stack = []
        self._handled_children = []

        if transform_cache is None:
            transform_cache = TransformCache()
        self._transform_cache = transform_cache

    @property
    def handled_children(self):
        """ List of children of the current node that have already been
        handled.

        Nodes that manually process their children (as opposed to allowing
        drawing / mouse systems to handle them automatically) may append nodes
        to this list to prevent systems from handling them.
        """
        return self._handled_children

    @property
    def canvas(self):
        """ The Canvas that originated this SceneEvent
        """
        return self._canvas

    @property
    def viewbox(self):
        """ The current viewbox.
        """
        if len(self._viewbox_stack) > 0:
            return self._viewbox_stack[-1]
        else:
            return None

    @property
    def path(self):
        """ The path of Entities leading from the root SubScene to the
        current recipient of this Event.
        """
        return self._stack

    def push_node(self, node):
        """ Push an node on the stack. """
        self._stack.append(node)
        self._handled_children.append([])
        if id(node) in self._stack_ids:
            raise RuntimeError("Scenegraph cycle detected; cannot push %r" %
                               node)
        self._stack_ids.add(id(node))
        doc = node.document
        if doc is not None:
            self.push_document(doc)

    def pop_node(self):
        """ Pop an node from the stack. """
        ent = self._stack.pop(-1)
        self._handled_children.pop(-1)
        self._stack_ids.remove(id(ent))
        if ent.document is not None:
            assert ent.document == self.pop_document()
        return ent

    def push_viewbox(self, viewbox):
        self._viewbox_stack.append(viewbox)

    def pop_viewbox(self):
        return self._viewbox_stack.pop(-1)

    def push_document(self, doc):
        self._doc_stack.append(doc)

    def pop_document(self):
        return self._doc_stack.pop(-1)

    def push_viewport(self, viewport):
        """ Push a viewport (x, y, w, h) on the stack. It is the
        responsibility of the caller to ensure the given values are
        int. The viewport's origin is defined relative to the current
        viewport.
        """
        self.canvas.push_viewport(viewport)

    def pop_viewport(self):
        """ Pop a viewport from the stack.
        """
        return self.canvas.pop_viewport()

    def push_fbo(self, viewport, fbo, transform):
        """ Push an FBO on the stack, together with the new viewport.
        and the transform to the FBO.
        """
        self.canvas.push_fbo(viewport, fbo, transform)

    def pop_fbo(self):
        """ Pop an FBO from the stack.
        """
        return self.canvas.pop_fbo()

    #
    # Begin transforms
    #

    @property
    def document_cs(self):
        """ The node for the current document coordinate system. The
        coordinate system of this Node is used for making physical
        measurements--px, mm, in, etc.
        """
        if len(self._doc_stack) > 0:
            return self._doc_stack[-1]
        else:
            return self.canvas_cs

    @property
    def canvas_cs(self):
        """ The node for the current canvas coordinate system. This cs
        represents the logical pixels of the canvas being drawn, with the
        origin in upper-left, and the canvas (width, height) in the bottom
        right. This coordinate system is most often used for handling mouse
        input.
        """
        return self.canvas.canvas_cs

    @property
    def framebuffer_cs(self):
        """ The node for the current framebuffer coordinate system. This
        coordinate system corresponds to the physical pixels being rendered
        to, with the origin in lower-right, and the framebufer (width, height)
        in upper-left. It is used mainly for making antialiasing measurements.
        """
        return self.canvas.framebuffer_cs

    @property
    def render_cs(self):
        """ Return node for the normalized device coordinate system. This
        coordinate system is the obligatory output of GLSL vertex shaders,
        with (-1, -1) in bottom-left, and (1, 1) in top-right. This coordinate
        system is frequently used for rendering visuals because all vertices
        must ultimately be mapped here.
        """
        return self.canvas.render_cs

    @property
    def node_cs(self):
        """ The node at the top of the node stack.
        """
        return self._stack[-1]

    @property
    def visual_to_canvas(self):
        """ Transform mapping from visual local coordinate frame to canvas
        coordinate frame.
        """
        return self.node_transform(map_to=self.canvas_cs,
                                   map_from=self._stack[-1])

    @property
    def visual_to_document(self):
        """ Transform mapping from visual local coordinate frame to document
        coordinate frame.
        """
        return self.node_transform(map_to=self.document_cs,
                                   map_from=self._stack[-1])

    @visual_to_document.setter
    def visual_to_document(self, tr):
        raise RuntimeError("Cannot set transforms on SceneEvent.")

    @property
    def document_to_framebuffer(self):
        """ Transform mapping from document coordinate frame to the framebuffer
        (physical pixel) coordinate frame.
        """
        return self.node_transform(map_to=self.framebuffer_cs,
                                   map_from=self.document_cs)

    @document_to_framebuffer.setter
    def document_to_framebuffer(self, tr):
        raise RuntimeError("Cannot set transforms on SceneEvent.")

    @property
    def framebuffer_to_render(self):
        """ Transform mapping from pixel coordinate frame to rendering
        coordinate frame.
        """
        return self.node_transform(map_to=self.render_cs,
                                   map_from=self.framebuffer_cs)

    @framebuffer_to_render.setter
    def framebuffer_to_render(self, tr):
        raise RuntimeError("Cannot set transforms on SceneEvent.")

    @property
    def visual_to_framebuffer(self):
        """ Transform mapping from visual coordinate frame to the framebuffer
        (physical pixel) coordinate frame.
        """
        return self.node_transform(map_to=self.framebuffer_cs, 
                                   map_from=self._stack[-1])
        
    def get_full_transform(self):
        """ Return the transform that maps from the current node to
        normalized device coordinates within the current glViewport and
        FBO.

        This transform consists of the full_transform prepended by a
        correction for the current glViewport and/or FBO.

        Most entities will use this transform when drawing.
        """
        return self._transform_cache.get([e.transform for e in self._stack])

    @property
    def scene_transform(self):
        """ The transform that maps from the current node to the first
        scene in its ancestry.
        """
        if len(self._viewbox_stack) > 1:
            view = self._viewbox_stack[-1]
            return self.node_transform(map_to=view.scene)
        else:
            return None

    @property
    def view_transform(self):
        """ The transform that maps from the current node to the first
        viewbox in its ancestry.
        """
        if len(self._viewbox_stack) > 1:
            view = self._viewbox_stack[-1]
            return self.node_transform(map_to=view)
        else:
            return None

    def node_transform(self, map_to=None, map_from=None):
        """ Return the transform from *map_from* to *map_to*, using the
        current node stack to resolve parent ambiguities if needed.

        By default, *map_to* is the normalized device coordinate system,
        and *map_from* is the current top node on the stack.
        """
        if map_to is None:
            map_to = self.render_cs
        if map_from is None:
            map_from = self._stack[-1]
        fwd_path = self._node_path(map_from, map_to)

        if fwd_path[-1] is map_to:
            fwd_path = fwd_path[:-1]
            rev_path = []
        else:
            # If we have still not reached the end, try traversing from the
            # opposite end. Note the reversed order of start and end
            rev_path = self._node_path(start=map_to, end=map_from)
            if rev_path[-1] is map_from:
                fwd_path = []
                rev_path = rev_path[:-1]

            else:
                # Find earliest intersection of fwd and rev paths
                connected = False
                while fwd_path[-1] is rev_path[-1]:
                    connected = True
                    fwd_path = fwd_path[:-1]
                    rev_path = rev_path[:-1]

                if not connected:
                    raise RuntimeError("Unable to find unique path from %r to "
                                       "%r" % (map_from, map_to))

        # starting node must come _last_ in the transform chain
        fwd_path = fwd_path[::-1]
        transforms = ([e.transform.inverse for e in rev_path] +
                      [e.transform for e in fwd_path])
        return self._transform_cache.get(transforms)

    def _node_path(self, start, end):
        """
        Return the path of parents leading from *start* to *end*, using the
        node stack to resolve multi-parent branches.

        If *end* is never reached, then the path is assembled as far as
        possible and returned.
        """
        path = [start]

        # first, get parents directly from node
        node = start
        while id(node) not in self._stack_ids:
            if node is end or len(node.parents) != 1:
                return path
            node = node.parent
            path.append(node)

        # if we have not reached the end, follow _stack if possible.
        if path[-1] is not end:
            try:
                ind = self._stack.index(node)
                # copy stack onto path one node at a time
                while ind > 0 and path[-1] is not end:
                    ind -= 1
                    path.append(self._stack[ind])
            except IndexError:
                pass
        return path


class SceneDrawEvent(SceneEvent):
    def __init__(self, event, canvas, **kwargs):
        self.draw_event = event
        super(SceneDrawEvent, self).__init__(type='draw', canvas=canvas,
                                             **kwargs)


class SceneMouseEvent(SceneEvent):
    """ Represents a mouse event that occurred on a SceneCanvas. This event is
    delivered to all entities whose mouse interaction area is under the event.
    """
    def __init__(self, event, canvas, **kwargs):
        self.mouse_event = event
        super(SceneMouseEvent, self).__init__(type=event.type, canvas=canvas,
                                              **kwargs)

    @property
    def pos(self):
        """ The position of this event in the local coordinate system of the
        visual.
        """
        return self.map_from_canvas(self.mouse_event.pos)

    @property
    def last_event(self):
        """ The mouse event immediately prior to this one. This
        property is None when no mouse buttons are pressed.
        """
        if self.mouse_event.last_event is None:
            return None
        ev = self.copy()
        ev.mouse_event = self.mouse_event.last_event
        return ev

    @property
    def press_event(self):
        """ The mouse press event that initiated a mouse drag, if any.
        """
        if self.mouse_event.press_event is None:
            return None
        ev = self.copy()
        ev.mouse_event = self.mouse_event.press_event
        return ev

    @property
    def button(self):
        """ The button pressed or released on this event.
        """
        return self.mouse_event.button

    @property
    def buttons(self):
        """ A list of all buttons currently pressed on the mouse.
        """
        return self.mouse_event.buttons

    @property
    def delta(self):
        """ The increment by which the mouse wheel has moved.
        """
        return self.mouse_event.delta

    def copy(self):
        ev = self.__class__(self.mouse_event, self._canvas)
        ev._stack = self._stack[:]
        # ev._ra_stack = self._ra_stack[:]
        ev._viewbox_stack = self._viewbox_stack[:]
        return ev

    def map_to_canvas(self, obj):
        tr = self.node_transform(map_from=self.node_cs,
                                 map_to=self.canvas_cs)
        return tr.map(obj)

    def map_from_canvas(self, obj):
        tr = self.node_transform(map_from=self.canvas_cs,
                                 map_to=self.node_cs)
        return tr.map(obj)
